<?php
/**
 * Adaptive Sites GC CLI
 *
 * 1. Run `gc adaptive-sites list`                     List of all adaptive sites.
 * 2. Run `gc adaptive-sites import <id>`    Import site.
 *
 * @package Adaptive Sites
 * @since 1.4.0
 */

if ( class_exists( 'GC_CLI_Command' ) && ! class_exists( 'Adaptive_Sites_GC_CLI' ) ) :

	/**
	 * GC-Cli commands to manage Adaptive Starter Sites.
	 *
	 * @since 1.4.0
	 */
	class Adaptive_Sites_GC_CLI extends GC_CLI_Command {

		/**
		 * Site Data
		 *
		 * @var array
		 */
		protected $current_site_data;

		/**
		 * Process Batch
		 *
		 * ## EXAMPLES
		 *
		 *     $ gc adaptive-sites batch
		 *      Processing Site: http://example.com/
		 *      Batch Process Started..
		 *      ..
		 *
		 * @since 2.1.0
		 * @param  array $args        Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 */
		public function batch( $args, $assoc_args ) {

			GC_CLI::line( 'Processing Site: ' . site_url() );

			Adaptive_Sites_Batch_Processing::get_instance()->start_process();
		}

		/**
		 * Generates the list of all Adaptive Sites.
		 *
		 * ## OPTIONS
		 *
		 * [--per-page=<number>]
		 * : No of sites to show in the list. Default its showing 10 sites.
		 *
		 * [--search=<text>]
		 * : Show the sites from particular search term.
		 *
		 * [--category=<text>]
		 * : Show the site from the specific category.
		 *
		 * [--page-builder=<text>]
		 * : List the sites from the particular page builder.
		 *
		 * [--type=<text>]
		 * : List the sites from the particular site type.
		 *
		 * ## EXAMPLES
		 *
		 *     # List all the sites.
		 *     $ gc adaptive-sites list
		 *     +-------+-------------------+-----------------------------------------+---------+----------------+--------------+
		 *     | id    | title             | url                                     | type    | categories     | page-builder |
		 *     +-------+-------------------+-----------------------------------------+---------+----------------+--------------+
		 *     | 34184 | Nutritionist      | //sitedemos.gechiui.com/nutritionist-01      | free    | Business, Free | Elementor    |
		 *     | 34055 | Law Firm          | //sitedemos.gechiui.com/law-firm-03          | premium | Business       | Elementor    |
		 *     +-------+-------------------+-----------------------------------------+---------+----------------+--------------+
		 *
		 * @since 1.4.0
		 * @param  array $args        Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 *
		 * @alias list
		 */
		public function list_sites( $args, $assoc_args ) {

			$per_page = isset( $assoc_args['per-page'] ) ? $assoc_args['per-page'] : 10;
			$search   = isset( $assoc_args['search'] ) ? $assoc_args['search'] : '';

			$rest_args = array(
				'_fields'  => 'id,title,slug,adaptive-site-category,adaptive-site-page-builder,adaptive-sites-tag,adaptive-site-type,adaptive-site-url',
				'per_page' => $per_page,
			);

			if ( ! empty( $search ) ) {
				$rest_args['search'] = $search;
			}

			$list = (array) $this->get_sites( 'adaptive-sites', $rest_args, true, $assoc_args );

			// Modify the output.
			foreach ( $list as $key => $item ) {
				$list[ $key ]['categories']   = implode( ', ', $list[ $key ]['categories'] );
				$list[ $key ]['page-builder'] = implode( ', ', $list[ $key ]['page_builders'] );
			}

			if ( ! empty( $list ) ) {
				$display_fields = array(
					'id',
					'title',
					'url',
					'type',
					'categories',
					'page-builder',
				);
				$formatter      = $this->get_formatter( $assoc_args, $display_fields );
				$formatter->display_items( $list );
			} else {
				GC_CLI::error( __( '没有发现网站!尝试另一个查询。', 'adaptive-sites' ) );
			}
		}

		/**
		 * Import the site by site ID.
		 *
		 * ## OPTIONS
		 *
		 * <id>
		 * : Site id of the import site.
		 *
		 * [--reset]
		 * : Reset the recently imported site data. Including post, pages, customizer settings, widgets etc.
		 *
		 * [--yes]
		 * : Forcefully import the site without asking any prompt message.
		 *
		 * ## EXAMPLES
		 *
		 *     # Import demo site.
		 *     $ gc adaptive-sites import 34184 --reset --yes
		 *     Activating Plugins..
		 *     Reseting Posts..
		 *     ..
		 *
		 * @since 1.4.0
		 * @param  array $args        Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 */
		public function import( $args, $assoc_args ) {

			// Force import.
			$yes = isset( $assoc_args['yes'] ) ? true : false;
			if ( ! $yes ) {
				GC_CLI::confirm( __( '您确定要导入站点吗?', 'adaptive-sites' ) );
			}

			// Valid site ID?
			$id = isset( $args[0] ) ? absint( $args[0] ) : 0;
			if ( ! $id ) {
				GC_CLI::error( __( '无效的站点ID,', 'adaptive-sites' ) );
			}

			$reset     = isset( $assoc_args['reset'] ) ? true : false;
			$site_url  = get_site_url();
			$demo_data = $this->get_site_data( $id );

			// Invalid Site ID.
			if ( is_gc_error( $demo_data ) ) {
				/* Translators: %s is the error message. */
				GC_CLI::error( sprintf( __( '由于错误:%s，站点导入失败', 'adaptive-sites' ), $demo_data->get_error_message() ) );
			}

			// License Status.
			// gongenlin
			$license_status = get_pro_license_valid();

			if ( 'free' !== $demo_data['site-type'] && 'upgrade' === $demo_data['license-status'] && ! $license_status ) {

				if ( ! defined( 'ADAPTIVE_SITES_NAME' ) ) {
					GC_CLI::line( __( 'This is Premium site. Please activate the "导入演示数据" license!', 'adaptive-sites' ) );
					GC_CLI::line( __( '使用”gc plugin deactivate adaptive-sites”，然后使用”gc plugin activate adaptive-sites”', 'adaptive-sites' ) );
				}

				/* translators: %s is the activate plugin license link. */
				GC_CLI::error( __( '使用 CLI 命令“gc license activate adaptive-sites{YOUR_LICENSE_KEY}”', 'adaptive-sites' ) );
			}

			/**
			 * Install & Activate Required Plugins.
			 */
			if ( isset( $demo_data['required-plugins'] ) ) {
				$plugins = (array) $demo_data['required-plugins'];
				if ( ! empty( $plugins ) ) {
					$plugin_status = Adaptive_Sites::get_instance()->required_plugin( $plugins, $demo_data['adaptive-site-options-data'], $demo_data['adaptive-enabled-extensions'] );

					// Install Plugins.
					if ( ! empty( $plugin_status['required_plugins']['notinstalled'] ) ) {
						GC_CLI::line( __( '开始安装插件。', 'adaptive-sites' ) );
						foreach ( $plugin_status['required_plugins']['notinstalled'] as $key => $plugin ) {
							if ( isset( $plugin['slug'] ) ) {

								// Install plugin.
								GC_CLI::runcommand( 'plugin install ' . $plugin['slug'] );

								// Activate plugin.
								Adaptive_Sites::get_instance()->required_plugin_activate( $plugin['init'], $demo_data['adaptive-site-options-data'], $demo_data['adaptive-enabled-extensions'] );
							}
						}
					}

					// Activate Plugins.
					if ( ! empty( $plugin_status['required_plugins']['inactive'] ) ) {
						GC_CLI::line( __( '开始激活插件..', 'adaptive-sites' ) );
						foreach ( $plugin_status['required_plugins']['inactive'] as $key => $plugin ) {
							if ( isset( $plugin['init'] ) ) {
								Adaptive_Sites::get_instance()->required_plugin_activate( $plugin['init'], $demo_data['adaptive-site-options-data'], $demo_data['adaptive-enabled-extensions'] );
							}
						}
					}
				}
			}

			/**
			 * Backup Customizer Settings
			 */
			Adaptive_Sites::get_instance()->backup_settings();

			/**
			 * Reset Site Data
			 */
			if ( $reset ) {
				GC_CLI::runcommand( 'adaptive-sites reset --yes' );
			}

			/**
			 * Import Flows & Steps for CartFlows.
			 */
			if ( isset( $demo_data['adaptive-site-cartflows-path'] ) && ! empty( $demo_data['adaptive-site-cartflows-path'] ) ) {
				Adaptive_Sites_Importer::get_instance()->import_cartflows( $demo_data['adaptive-site-cartflows-path'] );
			}

			/**
			 * Import GC Forms.
			 */
			if ( isset( $demo_data['adaptive-site-gcforms-path'] ) && ! empty( $demo_data['adaptive-site-gcforms-path'] ) ) {
				Adaptive_Sites_Importer::get_instance()->import_gcforms( $demo_data['adaptive-site-gcforms-path'] );
			}

			/**
			 * Import Customizer Settings.
			 */
			GC_CLI::runcommand( 'adaptive-sites import_customizer_settings ' . $id );

			/**
			 * Import Content from XML/WXR.
			 */
			if ( isset( $demo_data['adaptive-site-wxr-path'] ) && ! empty( $demo_data['adaptive-site-wxr-path'] ) ) {
				GC_CLI::runcommand( 'adaptive-sites import_wxr ' . $demo_data['adaptive-site-wxr-path'] );
			}

			/**
			 * Import Site Options.
			 */
			if ( isset( $demo_data['adaptive-site-options-data'] ) && ! empty( $demo_data['adaptive-site-options-data'] ) ) {
				GC_CLI::line( __( '导入站点选择. .', 'adaptive-sites' ) );
				Adaptive_Sites_Importer::get_instance()->import_options( $demo_data['adaptive-site-options-data'] );
			}

			/**
			 * Import Widgets.
			 */
			if ( isset( $demo_data['adaptive-site-widgets-data'] ) && ! empty( $demo_data['adaptive-site-widgets-data'] ) ) {
				GC_CLI::line( __( '导入小工具..', 'adaptive-sites' ) );
				Adaptive_Sites_Importer::get_instance()->import_widgets( $demo_data['adaptive-site-widgets-data'] );
			}

			/**
			 * Import End.
			 */
			GC_CLI::runcommand( 'adaptive-sites import_end' );

			/* translators: %s is the site URL. */
			GC_CLI::line( sprintf( __( "网站导入成功!\n访问:%s", 'adaptive-sites' ), $site_url ) );
		}

		/**
		 * Import End
		 *
		 * @since 1.4.3
		 * @return void
		 */
		public function import_end() {
			Adaptive_Sites_Importer::get_instance()->import_end();
		}

		/**
		 * Import form XML.
		 *
		 * ## OPTIONS
		 *
		 * <url>
		 * : XML/WXR file URL.
		 *
		 * ## EXAMPLES
		 *
		 *      $ gc adaptive-sites import_wxr <url>
		 *
		 * @since 1.4.3
		 * @param  array $args       Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 * @return void.
		 */
		public function import_wxr( $args = array(), $assoc_args = array() ) {

			// Valid site ID?
			$url = isset( $args[0] ) ? esc_url_raw( $args[0] ) : '';
			if ( empty( $url ) ) {
				GC_CLI::error( esc_html__( '无效的XML的URL。', 'adaptive-sites' ) );
			}

			// Download XML file.
			/* translators: %s is the XML file URL. */
			GC_CLI::line( sprintf( esc_html__( '下载%s', 'adaptive-sites' ), $url ) );
			$xml_path = Adaptive_Sites_Helper::download_file( $url );

			if ( $xml_path['success'] && isset( $xml_path['data']['file'] ) ) {
				GC_CLI::line( esc_html__( '导入WXR..', 'adaptive-sites' ) );
				Adaptive_WXR_Importer::instance()->sse_import( $xml_path['data']['file'] );
			} else {
				/* translators: %s is error message. */
				GC_CLI::line( printf( esc_html__( '下载WXR文件失败。错误%s', 'adaptive-sites' ), esc_html( $xml_path['data'] ) ) );
			}
		}

		/**
		 * Reset
		 *
		 * Delete all pages, post, custom post type, customizer settings and site options.
		 *
		 * ## OPTIONS
		 *
		 * [--yes]
		 * : Reset previously imported site data without asking the prompt message.
		 *
		 * ## EXAMPLES
		 *
		 *      $ gc adaptive-sites reset
		 *
		 * @since 1.4.0
		 * @param  array $args       Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 * @return void.
		 */
		public function reset( $args = array(), $assoc_args = array() ) {

			$yes = isset( $assoc_args['yes'] ) ? true : false;
			if ( ! $yes ) {
				GC_CLI::confirm( __( '确实要删除导入的站点数据吗?', 'adaptive-sites' ) );
			}

			// Get tracked data.
			$reset_data = Adaptive_Sites::get_instance()->get_reset_data();

			// Delete tracked posts.
			if ( isset( $reset_data['reset_posts'] ) && ! empty( $reset_data['reset_posts'] ) ) {
				GC_CLI::line( __( '重置的帖子. .', 'adaptive-sites' ) );
				foreach ( $reset_data['reset_posts'] as $key => $post_id ) {
					Adaptive_Sites_Importer::get_instance()->delete_imported_posts( $post_id );
				}
			}
			// Delete tracked terms.
			if ( isset( $reset_data['reset_terms'] ) && ! empty( $reset_data['reset_terms'] ) ) {
				GC_CLI::line( __( '重置条款。', 'adaptive-sites' ) );
				foreach ( $reset_data['reset_terms'] as $key => $post_id ) {
					Adaptive_Sites_Importer::get_instance()->delete_imported_terms( $post_id );
				}
			}
			// Delete tracked GC forms.
			if ( isset( $reset_data['reset_gc_forms'] ) && ! empty( $reset_data['reset_gc_forms'] ) ) {
				GC_CLI::line( __( '重置万能表单...', 'adaptive-sites' ) );
				foreach ( $reset_data['reset_gc_forms'] as $key => $post_id ) {
					Adaptive_Sites_Importer::get_instance()->delete_imported_gc_forms( $post_id );
				}
			}

			// Delete Customizer Data.
			Adaptive_Sites_Importer::get_instance()->reset_customizer_data();

			// Delete Site Options.
			Adaptive_Sites_Importer::get_instance()->reset_site_options();

			// Delete Widgets Data.
			Adaptive_Sites_Importer::get_instance()->reset_widgets_data();
		}

		/**
		 * Import Customizer Settings
		 *
		 * ## OPTIONS
		 *
		 * <id>
		 * : Site ID.
		 *
		 * ## EXAMPLES
		 *
		 *      $ gc adaptive-sites import_customizer_settings <id>
		 *
		 * @since 1.4.0
		 *
		 * @param  array $args        Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 * @return void
		 */
		public function import_customizer_settings( $args, $assoc_args ) {

			// Valid site ID?
			$id = isset( $args[0] ) ? absint( $args[0] ) : 0;
			if ( ! $id ) {
				GC_CLI::error( __( '无效的站点ID,', 'adaptive-sites' ) );
			}

			$demo_data = $this->get_site_data( $id );

			GC_CLI::line( __( '导入定制设置…', 'adaptive-sites' ) );
			Adaptive_Sites_Importer::get_instance()->import_customizer_settings( $demo_data['adaptive-site-customizer-data'] );
		}

		/**
		 * Page Builders
		 *
		 * ### OPTIONS
		 *
		 * [<list>]
		 * : List all page builders.
		 *
		 * OR
		 *
		 * [<set>]
		 * : Set the current page builder with given page builder slug.
		 *
		 * [<slug>]
		 * : Page builder slug.
		 *
		 * ### EXAMPLES
		 *
		 *     # List all the page builders.
		 *     λ gc adaptive-sites page_builder list
		 *     +----------------+----------------+
		 *     | slug           | name           |
		 *     +----------------+----------------+
		 *     | gutenberg      | Gutenberg      |
		 *     | elementor      | Elementor      |
		 *     | beaver-builder | Beaver Builder |
		 *     | brizy          | Brizy          |
		 *     +----------------+----------------+
		 *
		 *     # Set `Elementor` as default page builder.
		 *     λ gc adaptive-sites page_builder set elementor
		 *     "Elementor" is set as default page builder.
		 *
		 *     # Set `Beaver Builder` as default page builder.
		 *     λ gc adaptive-sites page_builder set beaver-builder
		 *     "Beaver Builder" is set as default page builder.
		 *
		 * @since 1.4.0
		 * @param  array $args        Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 */
		public function page_builder( $args, $assoc_args ) {
			$action = isset( $args[0] ) ? $args[0] : '';

			if ( empty( $action ) ) {
				GC_CLI::error( __( '请添加有效的参数。', 'adaptive-sites' ) );
			}

			$page_builders = Adaptive_Sites_Page::get_instance()->get_page_builders();

			if ( 'list' === $action ) {
				$display_fields = array(
					'slug',
					'name',
				);
				$formatter      = $this->get_formatter( $assoc_args, $display_fields );
				$formatter->display_items( $page_builders );

				$default_page_builder = isset( $page_builders[ Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' ) ] ) ? $page_builders[ Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' ) ]['name'] : '';

				if ( ! empty( $default_page_builder ) ) {
					/* translators: %s is the current page builder name. */
					GC_CLI::line( sprintf( __( '默认的页面构建器是"%s"。', 'adaptive-sites' ), $default_page_builder ) );
				}
			} elseif ( 'set' === $action ) {
				$page_builder_slugs = array_keys( $page_builders );
				$page_builder_slug  = isset( $args[1] ) ? $args[1] : '';
				if ( in_array( $page_builder_slug, $page_builder_slugs, true ) ) {
					Adaptive_Sites_Page::get_instance()->save_page_builder_on_submit( $page_builder_slug );
					/* translators: %s is the page builder name. */
					GC_CLI::line( sprintf( __( '“%s”被设置为默认的页面构建器。', 'adaptive-sites' ), $page_builders[ $page_builder_slug ]['name'] ) );

				} else {
					GC_CLI::error( __( "无效的页面构建程序段塞。\n用命令“gc adaptive-sites page_builder list”检查所有元素构建器错误", 'adaptive-sites' ) );
				}
			} else {
				GC_CLI::error( __( "无效的参数!\n请使用“列表”或“设置”参数。", 'adaptive-sites' ) );
			}
		}

		/**
		 * Get Formatter
		 *
		 * @since 1.4.0
		 * @param  array  $assoc_args Associate arguments.
		 * @param  string $fields    Fields.
		 * @param  string $prefix    Prefix.
		 * @return object            Class object.
		 */
		protected function get_formatter( &$assoc_args, $fields = '', $prefix = '' ) {
			return new \GC_CLI\Formatter( $assoc_args, $fields, $prefix );
		}

		/**
		 * Get Site Data by Site ID
		 *
		 * @since 1.4.0
		 *
		 * @param  int $id        Site ID.
		 * @return array
		 */
		private function get_site_data( $id ) {
			if ( empty( $this->current_site_data ) ) {
				// @todo Use Adaptive_Sites::get_instance()->api_request() instead of below function.
				$this->current_site_data = Adaptive_Sites_Importer::get_instance()->get_single_demo( $id );
				update_option( 'adaptive_sites_import_data', $this->current_site_data, 'no' );
			}

			return $this->current_site_data;
		}

		/**
		 * Get Sites
		 *
		 * @since 1.4.0
		 *
		 * @param  string  $post_slug  Post slug.
		 * @param  array   $args       Post query arguments.
		 * @param  boolean $force      Force import.
		 * @param  array   $assoc_args Associate arguments.
		 * @return array
		 */
		private function get_sites( $post_slug = '', $args = array(), $force = false, $assoc_args = array() ) {

			// Add page builders.
			$page_builder  = isset( $assoc_args['page-builder'] ) ? $assoc_args['page-builder'] : Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' );
			$response      = $this->get_term_ids( 'adaptive-site-page-builder', $page_builder, $args );
			$args          = $response['args'];
			$page_builders = $response['terms'];
			if ( empty( $page_builders['data'] ) ) {
				GC_CLI::error( __( '这个元素构建器插件没有安装。请尝试不同的页面构建器。', 'adaptive-sites' ) );
			}

			// Add type.
			$type     = isset( $assoc_args['type'] ) ? $assoc_args['type'] : '';
			$response = $this->get_term_ids( 'adaptive-sites-type', $type, $args );
			$args     = $response['args'];
			$types    = $response['terms'];
			if ( empty( $types['data'] ) ) {
				GC_CLI::error( __( '此站点类型不存在。请尝试不同的网站类型。', 'adaptive-sites' ) );
			}

			// Add categories.
			$category   = isset( $assoc_args['category'] ) ? $assoc_args['category'] : '';
			$response   = $this->get_term_ids( 'adaptive-site-category', $category, $args );
			$args       = $response['args'];
			$categories = $response['terms'];
			if ( empty( $categories['data'] ) ) {
				GC_CLI::error( __( '此网站类别不存在。请尝试一个不同的网站类别。', 'adaptive-sites' ) );
			}

			// Site list.
			$sites = (array) $this->get_posts( 'adaptive-sites', $args, $force );

			$list = array();
			if ( $sites['success'] ) {
				foreach ( $sites['data'] as $key => $site ) {
					$single_site = array(
						'id'            => $site['id'],
						'slug'          => $site['slug'],
						'title'         => $site['title']['rendered'],
						'url'           => $site['adaptive-site-url'],
						'type'          => ( 'premium' === $site['adaptive-site-type'] ) ? 'Premium' : ucwords( $site['adaptive-site-type'] ),
						'categories'    => array(),
						'page_builders' => array(),
					);

					if ( isset( $site['adaptive-site-category'] ) && ! empty( $categories['data'] ) ) {
						foreach ( $site['adaptive-site-category'] as $category_key => $category_id ) {
							if ( isset( $categories['data'][ $category_id ] ) ) {
								$single_site['categories'][ $category_id ] = $categories['data'][ $category_id ];
							}
						}
					}

					if ( isset( $site['adaptive-site-page-builder'] ) && ! empty( $page_builders['data'] ) ) {
						foreach ( $site['adaptive-site-page-builder'] as $page_builder_key => $page_builder_id ) {
							if ( isset( $page_builders['data'][ $page_builder_id ] ) ) {
								$single_site['page_builders'][ $page_builder_id ] = $page_builders['data'][ $page_builder_id ];
							}
						}
					}

					$list[] = $single_site;
				}
			}

			return $list;
		}


		/**
		 * Get Term IDs
		 *
		 * @since 1.4.0
		 *
		 * @param  string $term_slug   Term slug.
		 * @param  string $search_term Search term.
		 * @param  array  $args        Term query arguments.
		 * @return array               Term response.
		 */
		private function get_term_ids( $term_slug = '', $search_term = '', $args = array() ) {
			$term_args = array();

			if ( ! empty( $search_term ) ) {
				$term_args = array(
					'search' => $search_term,
				);
			}

			$term_response = (array) $this->get_terms( $term_slug, $term_args, true );

			if ( ! empty( $search_term ) ) {
				if ( ! empty( $term_response ) && is_array( $term_response['data'] ) ) {
					$args[ $term_slug ] = implode( ',', array_keys( $term_response['data'] ) );
				}
			}

			return array(
				'args'  => $args,
				'terms' => $term_response,
			);
		}

		/**
		 * Get Terms
		 *
		 * @since 1.0.0
		 *
		 * @param  array  $term_slug Term Slug.
		 * @param  array  $args      For selecting the demos (Search terms, pagination etc).
		 * @param  string $force     Force import.
		 * @return $array            Term response.
		 */
		private function get_terms( $term_slug = '', $args = array(), $force = false ) {

			$defaults = array(
				'_fields' => 'id,name,slug,count',
			);
			$args     = gc_parse_args( (array) $args, $defaults );

			$success    = false;
			$terms_data = get_transient( 'adaptive-sites-term-' . $term_slug );
			if ( empty( $terms_data ) || $force ) {
				$url = add_query_arg( $args, Adaptive_Sites::get_instance()->get_api_url() . $term_slug );

				$api_args = array(
					'timeout' => 60,
				);

				$response = gc_remote_get( $url, $api_args );
				if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
					$request_term_data = json_decode( gc_remote_retrieve_body( $response ), true );

					if ( ! isset( $request_term_data['code'] ) ) {
						$success        = true;
						$new_terms_data = array();
						foreach ( $request_term_data as $key => $request_term ) {
							$new_terms_data[ $request_term['id'] ] = $request_term['name'];
						}
						if ( set_transient( 'adaptive-sites-term-' . $term_slug, $new_terms_data, WEEK_IN_SECONDS ) ) {
							return array(
								'success' => $success,
								'data'    => $new_terms_data,
							);
						}
					}
				}
			}

			return array(
				'success' => $success,
				'data'    => $terms_data,
			);
		}

		/**
		 * Get Posts
		 *
		 * @since 1.4.0
		 *
		 * @param  string  $post_slug  Post slug.
		 * @param  array   $args       Post query arguments.
		 * @param  boolean $force      Force import.
		 * @return array
		 */
		private function get_posts( $post_slug = '', $args = array(), $force = false ) {

			$args = gc_parse_args( (array) $args, array() );

			$all_posts = get_transient( 'adaptive-sites-post-' . $post_slug );

			if ( empty( $all_posts ) || $force ) {
				$url = add_query_arg( $args, Adaptive_Sites::get_instance()->get_api_url() . $post_slug );

				$api_args = array(
					'timeout' => 60,
				);

				$success  = false;
				$response = gc_remote_get( $url, $api_args );
				if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
					$all_posts = json_decode( gc_remote_retrieve_body( $response ), true );

					if ( ! isset( $all_posts['code'] ) ) {
						$success = true;
						set_transient( 'adaptive-sites-post-' . $post_slug, $all_posts, WEEK_IN_SECONDS );
					}
				}
			} else {
				$success = true;
			}

			return array(
				'success' => $success,
				'data'    => $all_posts,
			);
		}

		/**
		 * Sync Library.
		 *
		 * Sync the library and create the .json files.
		 *
		 * Use: `gc adaptive-sites sync`
		 *
		 * @since 2.0.0
		 * @param  array $args       Arguments.
		 * @param  array $assoc_args Associated Arguments.
		 * @return void.
		 */
		public function sync( $args = array(), $assoc_args = array() ) {
			Adaptive_Sites_Batch_Processing::get_instance()->process_batch();
		}
	}

	/**
	 * Add Command
	 */
	GC_CLI::add_command( 'starter-templates', 'Adaptive_Sites_GC_CLI' );
	GC_CLI::add_command( 'adaptive-sites', 'Adaptive_Sites_GC_CLI' );

endif;
