<?php
/**
 * Adaptive Sites Importer
 *
 * @since  1.0.0
 * @package Adaptive Sites
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'Adaptive_Sites_Importer' ) ) {

	/**
	 * Adaptive Sites Importer
	 */
	class Adaptive_Sites_Importer {

		/**
		 * Instance
		 *
		 * @since  1.0.0
		 * @var (Object) Class object
		 */
		public static $instance = null;

		/**
		 * Set Instance
		 *
		 * @since  1.0.0
		 *
		 * @return object Class object.
		 */
		public static function get_instance() {
			if ( ! isset( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * Constructor.
		 *
		 * @since  1.0.0
		 */
		public function __construct() {

			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-importer-log.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/importers/class-adaptive-sites-helper.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/importers/class-adaptive-widget-importer.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/importers/class-adaptive-customizer-import.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/importers/class-adaptive-site-options-import.php';

			// Import AJAX.
			add_action( 'gc_ajax_adaptive-sites-import-gcforms', array( $this, 'import_gcforms' ) );
			add_action( 'gc_ajax_adaptive-sites-import-cartflows', array( $this, 'import_cartflows' ) );
			add_action( 'gc_ajax_adaptive-sites-import-customizer-settings', array( $this, 'import_customizer_settings' ) );
			add_action( 'gc_ajax_adaptive-sites-import-prepare-xml', array( $this, 'prepare_xml_data' ) );
			add_action( 'gc_ajax_adaptive-sites-import-options', array( $this, 'import_options' ) );
			add_action( 'gc_ajax_adaptive-sites-import-widgets', array( $this, 'import_widgets' ) );
			add_action( 'gc_ajax_adaptive-sites-import-end', array( $this, 'import_end' ) );

			// Hooks in AJAX.
			add_action( 'adaptive_sites_import_complete', array( $this, 'after_batch_complete' ) );
			add_action( 'init', array( $this, 'load_importer' ) );

			require_once ADAPTIVE_SITES_DIR . 'inc/importers/batch-processing/class-adaptive-sites-batch-processing.php';

			add_action( 'adaptive_sites_image_import_complete', array( $this, 'after_batch_complete' ) );

			// Reset Customizer Data.
			add_action( 'gc_ajax_adaptive-sites-reset-customizer-data', array( $this, 'reset_customizer_data' ) );
			add_action( 'gc_ajax_adaptive-sites-reset-site-options', array( $this, 'reset_site_options' ) );
			add_action( 'gc_ajax_adaptive-sites-reset-widgets-data', array( $this, 'reset_widgets_data' ) );

			// Reset Post & Terms.
			add_action( 'gc_ajax_adaptive-sites-delete-posts', array( $this, 'delete_imported_posts' ) );
			add_action( 'gc_ajax_adaptive-sites-delete-gc-forms', array( $this, 'delete_imported_gc_forms' ) );
			add_action( 'gc_ajax_adaptive-sites-delete-terms', array( $this, 'delete_imported_terms' ) );

			if ( version_compare( get_bloginfo( 'version' ), '5.1.0', '>=' ) ) {
				add_filter( 'http_request_timeout', array( $this, 'set_timeout_for_images' ), 10, 2 );
			}
		}

		/**
		 * Set the timeout for the HTTP request by request URL.
		 *
		 * E.g. If URL is images (jpg|png|gif|jpeg) are from the domain `https://sitedemos.gechiui.com` then we have set the timeout by 30 seconds. Default 5 seconds.
		 *
		 * @since 1.3.8
		 *
		 * @param int    $timeout_value Time in seconds until a request times out. Default 5.
		 * @param string $url           The request URL.
		 */
		public function set_timeout_for_images( $timeout_value, $url ) {

			// URL not contain `https://sitedemos.gechiui.com` then return $timeout_value.
			if ( strpos( $url, 'https://sitedemos.gechiui.com' ) === false ) {
				return $timeout_value;
			}

			// Check is image URL of type jpg|png|gif|jpeg.
			if ( adaptive_sites_is_valid_image( $url ) ) {
				$timeout_value = 300;
			}

			return $timeout_value;
		}

		/**
		 * Load GeChiUI WXR importer.
		 */
		public function load_importer() {
			require_once ADAPTIVE_SITES_DIR . 'inc/importers/wxr-importer/class-adaptive-wxr-importer.php';
		}

		/**
		 * Change flow status
		 *
		 * @since 2.0.0
		 *
		 * @param  array $args Flow query args.
		 * @return array Flow query args.
		 */
		public function change_flow_status( $args ) {
			$args['post_status'] = 'publish';
			return $args;
		}

		/**
		 * Track Flow
		 *
		 * @since 2.0.0
		 *
		 * @param  integer $flow_id Flow ID.
		 * @return void
		 */
		public function track_flows( $flow_id ) {
			adaptive_sites_error_log( 'Flow ID ' . $flow_id );
			Adaptive_WXR_Importer::instance()->track_post( $flow_id );
		}

		/**
		 * Import GC Forms
		 *
		 * @since 1.2.14
		 * @since 1.4.0 The `$gcforms_url` was added.
		 *
		 * @param  string $gcforms_url GC Forms JSON file URL.
		 * @return void
		 */
		public function import_gcforms( $gcforms_url = '' ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$gcforms_url = ( isset( $_REQUEST['gcforms_url'] ) ) ? urldecode( $_REQUEST['gcforms_url'] ) : $gcforms_url;
			$ids_mapping = array();

			if ( ! empty( $gcforms_url ) && function_exists( 'gcforms_encode' ) ) {

				// Download JSON file.
				$file_path = Adaptive_Sites_Helper::download_file( $gcforms_url );

				if ( $file_path['success'] ) {
					if ( isset( $file_path['data']['file'] ) ) {

						$ext = strtolower( pathinfo( $file_path['data']['file'], PATHINFO_EXTENSION ) );

						if ( 'json' === $ext ) {
							$forms = json_decode( Adaptive_Sites::get_instance()->get_filesystem()->get_contents( $file_path['data']['file'] ), true );

							if ( ! empty( $forms ) ) {

								foreach ( $forms as $form ) {
									$title = ! empty( $form['settings']['form_title'] ) ? $form['settings']['form_title'] : '';
									// gongenlin
									$slug = ! empty( $form['settings']['form_slug'] ) ? $form['settings']['form_slug'] : null;
									$desc  = ! empty( $form['settings']['form_desc'] ) ? $form['settings']['form_desc'] : '';
									// gongenlin
									// 因为post_name可能重复，导致post_name与form_slug不一致。因此删除用于确保唯一性
									unset( $form['settings']['form_slug'] );

									$new_id = post_exists( $title );

									if ( ! $new_id ) {
										$new_id = gc_insert_post(
											array(
												'post_title'   => $title,
												'post_name'    => $slug, // gongenlin
												'post_status'  => 'publish',
												'post_type'    => 'gcforms',
												'post_excerpt' => $desc,
											)
										);

										if ( defined( 'GC_CLI' ) ) {
											GC_CLI::line( 'Imported Form ' . $title );
										}

										// Set meta for tracking the post.
										update_post_meta( $new_id, '_adaptive_sites_imported_gc_forms', true );
										Adaptive_Sites_Importer_Log::add( 'Inserted GC Form ' . $new_id );
									}

									if ( $new_id ) {

										// ID mapping.
										$ids_mapping[ $form['id'] ] = $new_id;

										$form['id'] = $new_id;
										gc_update_post(
											array(
												'ID' => $new_id,
												'post_content' => gcforms_encode( $form ),
											)
										);
									}
								}
							}
						}
					}
				}
			}

			update_option( 'adaptive_sites_gcforms_ids_mapping', $ids_mapping, 'no' );

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( 'GC Forms Imported.' );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success( $ids_mapping );
			}
		}

		/**
		 * Import CartFlows
		 *
		 * @since 2.0.0
		 *
		 * @param  string $url Cartflows JSON file URL.
		 * @return void
		 */
		public function import_cartflows( $url = '' ) {

			// Make the flow publish.
			add_action( 'cartflows_flow_importer_args', array( $this, 'change_flow_status' ) );
			add_action( 'cartflows_flow_imported', array( $this, 'track_flows' ) );
			add_action( 'cartflows_step_imported', array( $this, 'track_flows' ) );

			$url = ( isset( $_REQUEST['cartflows_url'] ) ) ? urldecode( $_REQUEST['cartflows_url'] ) : urldecode( $url ); // phpcs:ignore GeChiUI.Security.NonceVerification.Recommended
			if ( ! empty( $url ) && is_callable( 'CartFlows_Importer::get_instance' ) ) {

				// Download JSON file.
				$file_path = Adaptive_Sites_Helper::download_file( $url );

				if ( $file_path['success'] ) {
					if ( isset( $file_path['data']['file'] ) ) {

						$ext = strtolower( pathinfo( $file_path['data']['file'], PATHINFO_EXTENSION ) );

						if ( 'json' === $ext ) {
							$flows = json_decode( Adaptive_Sites::get_instance()->get_filesystem()->get_contents( $file_path['data']['file'] ), true );

							if ( ! empty( $flows ) ) {
								CartFlows_Importer::get_instance()->import_from_json_data( $flows );
							}
						}
					}
				}
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( 'Imported from ' . $url );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success( $url );
			}
		}

		/**
		 * Import Customizer Settings.
		 *
		 * @since 1.0.14
		 * @since 1.4.0  The `$customizer_data` was added.
		 *
		 * @param  array $customizer_data Customizer Data.
		 * @return void
		 */
		public function import_customizer_settings( $customizer_data = array() ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$customizer_data = adaptive_get_site_data( 'adaptive-site-customizer-data' );

			if ( ! empty( $customizer_data ) ) {

				Adaptive_Sites_Importer_Log::add( 'Imported Customizer Settings ' . gc_json_encode( $customizer_data ) );

				// Set meta for tracking the post.
				adaptive_sites_error_log( 'Customizer Data ' . gc_json_encode( $customizer_data ) );

				update_option( '_adaptive_sites_old_customizer_data', $customizer_data, 'no' );

				Adaptive_Customizer_Import::instance()->import( $customizer_data );

				if ( defined( 'GC_CLI' ) ) {
					GC_CLI::line( 'Imported Customizer Settings!' );
				} elseif ( gc_doing_ajax() ) {
					gc_send_json_success( $customizer_data );
				}
			} 

		}

		/**
		 * Prepare XML Data.
		 *
		 * @since 1.1.0
		 * @return void
		 */
		public function prepare_xml_data() {

			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'customize' ) ) {
				gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
			}

			if ( ! class_exists( 'XMLReader' ) ) {
				gc_send_json_error( __( '如果XMLReader不可用，它会导入所有其他设置，并且只跳过XML导入。这将创建一个不完整的网站。我们应该尽早bail，如果没有这些，就不要导入任何东西。', 'adaptive-sites' ) );
			}

			$wxr_url = ( isset( $_REQUEST['wxr_url'] ) ) ? urldecode( $_REQUEST['wxr_url'] ) : '';

			if ( isset( $wxr_url ) ) {

				Adaptive_Sites_Importer_Log::add( 'Importing from XML ' . $wxr_url );

				$overrides = array(
					'gc_handle_sideload' => 'upload',
				);

				// Download XML file.
				$xml_path = Adaptive_Sites_Helper::download_file( $wxr_url, $overrides );

				if ( $xml_path['success'] ) {

					$post = array(
						'post_title'     => basename( $wxr_url ),
						'guid'           => $xml_path['data']['url'],
						'post_mime_type' => $xml_path['data']['type'],
					);

					adaptive_sites_error_log( gc_json_encode( $post ) );
					adaptive_sites_error_log( gc_json_encode( $xml_path ) );

					// as per gc-admin/includes/upload.php.
					$post_id = gc_insert_attachment( $post, $xml_path['data']['file'] );

					adaptive_sites_error_log( gc_json_encode( $post_id ) );

					if ( is_gc_error( $post_id ) ) {
						gc_send_json_error( __( '下载XML文件时出现错误。', 'adaptive-sites' ) );
					} else {

						update_option( 'adaptive_sites_imported_wxr_id', $post_id, 'no' );
						$attachment_metadata = gc_generate_attachment_metadata( $post_id, $xml_path['data']['file'] );
						gc_update_attachment_metadata( $post_id, $attachment_metadata );
						$data        = Adaptive_WXR_Importer::instance()->get_xml_data( $xml_path['data']['file'], $post_id );
						$data['xml'] = $xml_path['data'];
						gc_send_json_success( $data );
					}
				} else {
					gc_send_json_error( $xml_path['data'] );
				}
			} else {
				gc_send_json_error( __( '无效的网站XML文件!', 'adaptive-sites' ) );
			}

		}

		/**
		 * Import Options.
		 *
		 * @since 1.0.14
		 * @since 1.4.0 The `$options_data` was added.
		 *
		 * @param  array $options_data Site Options.
		 * @return void
		 */
		public function import_options( $options_data = array() ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$options_data = adaptive_get_site_data( 'adaptive-site-options-data' );

			if ( ! empty( $options_data ) ) {
				// Set meta for tracking the post.
				if ( is_array( $options_data ) ) {
					Adaptive_Sites_Importer_Log::add( 'Imported - Site Options ' . gc_json_encode( $options_data ) );
					update_option( '_adaptive_sites_old_site_options', $options_data, 'no' );
				}

				$options_importer = Adaptive_Site_Options_Import::instance();
				$options_importer->import_options( $options_data );
				if ( defined( 'GC_CLI' ) ) {
					GC_CLI::line( 'Imported Site Options!' );
				} elseif ( gc_doing_ajax() ) {
					gc_send_json_success( $options_data );
				}
			} 

		}

		/**
		 * Import Widgets.
		 *
		 * @since 1.0.14
		 * @since 1.4.0 The `$widgets_data` was added.
		 *
		 * @param  string $widgets_data Widgets Data.
		 * @return void
		 */
		public function import_widgets( $widgets_data = '' ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$widgets_data = ( isset( $_POST['widgets_data'] ) ) ? (object) json_decode( stripslashes( $_POST['widgets_data'] ) ) : (object) $widgets_data;

			if ( ! empty( $widgets_data ) ) {

				Adaptive_Widget_Importer::instance()->import_widgets_data( $widgets_data );

				$sidebars_widgets = get_option( 'sidebars_widgets', array() );
				update_option( '_adaptive_sites_old_widgets_data', $sidebars_widgets, 'no' );
				Adaptive_Sites_Importer_Log::add( 'Imported - Widgets ' . gc_json_encode( $sidebars_widgets ) );

				if ( defined( 'GC_CLI' ) ) {
					GC_CLI::line( 'Widget Imported!' );
				} elseif ( gc_doing_ajax() ) {
					gc_send_json_success( $widgets_data );
				}
			} else {
				if ( defined( 'GC_CLI' ) ) {
					GC_CLI::line( '小工具数据为空!' );
				} elseif ( gc_doing_ajax() ) {
					gc_send_json_error( __( '小工具数据为空!', 'adaptive-sites' ) );
				}
			}

		}

		/**
		 * Import End.
		 *
		 * @since 1.0.14
		 * @return void
		 */
		public function import_end() {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$demo_data = get_option( 'adaptive_sites_import_data', array() );

			do_action( 'adaptive_sites_import_complete', $demo_data );

			update_option( 'adaptive_sites_import_complete', 'yes', 'no' );

			if ( gc_doing_ajax() ) {
				gc_send_json_success();
			}
		}

		/**
		 * Get single demo.
		 *
		 * @since  1.0.0
		 *
		 * @param  (String) $demo_api_uri API URL of a demo.
		 *
		 * @return (Array) $adaptive_demo_data demo data for the demo.
		 */
		public static function get_single_demo( $demo_api_uri ) {

			if ( is_int( $demo_api_uri ) ) {
				$demo_api_uri = Adaptive_Sites::get_instance()->get_api_url() . 'adaptive-sites/' . $demo_api_uri;
			}

			// default values.
			$remote_args = array();
			$defaults    = array(
				'id'                          => '',
				'adaptive-site-widgets-data'     => '',
				'adaptive-site-customizer-data'  => '',
				'adaptive-site-options-data'     => '',
				'adaptive-post-data-mapping'     => '',
				'adaptive-site-wxr-path'         => '',
				'adaptive-site-gcforms-path'     => '',
				'adaptive-enabled-extensions'    => '',
				'adaptive-custom-404'            => '',
				'required-plugins'            => '',
				'required-themes'            => '',
				'adaptive-site-taxonomy-mapping' => '',
				'license-status'              => '',
				'site-type'                   => '',
				'adaptive-site-url'              => '',
			);

			$api_args = apply_filters(
				'adaptive_sites_api_args',
				array(
					'timeout' => 15,
				)
			);

			// Use this for premium demos.
			$request_params = apply_filters(
				'adaptive_sites_api_params',
				array(
					'purchase_key' => '',
					'site_url'     => '',
				)
			);

			$demo_api_uri = add_query_arg( $request_params, $demo_api_uri );

			// API Call.
			$response = gc_remote_get( $demo_api_uri, $api_args );

			if ( is_gc_error( $response ) || ( isset( $response->status ) && 0 === $response->status ) ) {
				if ( isset( $response->status ) ) {
					$data = json_decode( $response, true );
				} else {
					return new GC_Error( 'api_invalid_response_code', $response->get_error_message() );
				}
			}

			if ( gc_remote_retrieve_response_code( $response ) !== 200 ) {
				return new GC_Error( 'api_invalid_response_code', gc_remote_retrieve_body( $response ) );
			} else {
				$data = json_decode( gc_remote_retrieve_body( $response ), true );
			}

			$data = json_decode( gc_remote_retrieve_body( $response ), true );

			if ( ! isset( $data['code'] ) ) {
				$remote_args['id']                          = $data['id'];
				$remote_args['adaptive-site-widgets-data']     = json_decode( $data['adaptive-site-widgets-data'] );
				$remote_args['adaptive-site-customizer-data']  = $data['adaptive-site-customizer-data'];
				$remote_args['adaptive-site-options-data']     = $data['adaptive-site-options-data'];
				$remote_args['adaptive-post-data-mapping']     = $data['adaptive-post-data-mapping'];
				$remote_args['adaptive-site-wxr-path']         = $data['adaptive-site-wxr-path'];
				$remote_args['adaptive-site-gcforms-path']     = $data['adaptive-site-gcforms-path'];
				$remote_args['adaptive-enabled-extensions']    = $data['adaptive-enabled-extensions'];
				$remote_args['adaptive-custom-404']            = $data['adaptive-custom-404'];
				$remote_args['required-plugins']            = $data['required-plugins'];
				$remote_args['required-themes']            = $data['required-themes'];
				$remote_args['adaptive-site-taxonomy-mapping'] = $data['adaptive-site-taxonomy-mapping'];
				$remote_args['license-status']              = $data['license-status'];
				$remote_args['site-type']                   = $data['adaptive-site-type'];
				$remote_args['adaptive-site-url']              = $data['adaptive-site-url'];
			}

			// Merge remote demo and defaults.
			return gc_parse_args( $remote_args, $defaults );
		}

		/**
		 * Clear Cache.
		 *
		 * @since  1.0.9
		 */
		public function after_batch_complete() {

			// Clear 'Builder Builder' cache.
			if ( is_callable( 'FLBuilderModel::delete_asset_cache_for_all_posts' ) ) {
				FLBuilderModel::delete_asset_cache_for_all_posts();
			}

			// Clear 'Adaptive Addon' cache.
			if ( is_callable( 'Adaptive_Minify::refresh_assets' ) ) {
				Adaptive_Minify::refresh_assets();
			}

			$this->update_latest_checksums();

			// Flush permalinks.
			flush_rewrite_rules();

			delete_option( 'adaptive_sites_import_data' );

			Adaptive_Sites_Importer_Log::add( 'Complete ' );
		}

		/**
		 * Update Latest Checksums
		 *
		 * Store latest checksum after batch complete.
		 *
		 * @since 2.0.0
		 * @return void
		 */
		public function update_latest_checksums() {
			$latest_checksums = get_site_option( 'adaptive-sites-last-export-checksums-latest', '' );
			update_site_option( 'adaptive-sites-last-export-checksums', $latest_checksums, 'no' );
		}

		/**
		 * Reset customizer data
		 *
		 * @since 1.3.0
		 * @return void
		 */
		public function reset_customizer_data() {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			Adaptive_Sites_Importer_Log::add( 'Deleted customizer Settings ' . gc_json_encode( get_option( 'adaptive-settings', array() ) ) );

			delete_option( 'adaptive-settings' );

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( 'Deleted Customizer Settings!' );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success();
			}
		}

		/**
		 * Reset site options
		 *
		 * @since 1.3.0
		 * @return void
		 */
		public function reset_site_options() {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$options = get_option( '_adaptive_sites_old_site_options', array() );

			Adaptive_Sites_Importer_Log::add( 'Deleted - Site Options ' . gc_json_encode( $options ) );

			if ( $options ) {
				foreach ( $options as $option_key => $option_value ) {
					delete_option( $option_key );
				}
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( 'Deleted Site Options!' );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success();
			}
		}

		/**
		 * Reset widgets data
		 *
		 * @since 1.3.0
		 * @return void
		 */
		public function reset_widgets_data() {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			// Get all old widget ids.
			$old_widgets_data = (array) get_option( '_adaptive_sites_old_widgets_data', array() );
			$old_widget_ids = array();
			foreach ( $old_widgets_data as $old_sidebar_key => $old_widgets ) {
				if ( ! empty( $old_widgets ) && is_array( $old_widgets ) ) {
					$old_widget_ids = array_merge( $old_widget_ids, $old_widgets );
				}
			}

			// Process if not empty.
			$sidebars_widgets = get_option( 'sidebars_widgets', array() );
			if ( ! empty( $old_widget_ids ) && ! empty( $sidebars_widgets ) ) {

				Adaptive_Sites_Importer_Log::add( 'DELETED - WIDGETS ' . gc_json_encode( $old_widget_ids ) );

				foreach ( $sidebars_widgets as $sidebar_id => $widgets ) {
					$widgets = (array) $widgets;

					if ( ! empty( $widgets ) && is_array( $widgets ) ) {
						foreach ( $widgets as $widget_id ) {

							if ( in_array( $widget_id, $old_widget_ids, true ) ) {
								Adaptive_Sites_Importer_Log::add( 'DELETED - WIDGET ' . $widget_id );

								// Move old widget to inacitve list.
								$sidebars_widgets['gc_inactive_widgets'][] = $widget_id;

								// Remove old widget from sidebar.
								$sidebars_widgets[ $sidebar_id ] = array_diff( $sidebars_widgets[ $sidebar_id ], array( $widget_id ) );
							}
						}
					}
				}

				update_option( 'sidebars_widgets', $sidebars_widgets );
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( 'Deleted Widgets!' );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success();
			}
		}

		/**
		 * Delete imported posts
		 *
		 * @since 1.3.0
		 * @since 1.4.0 The `$post_id` was added.
		 *
		 * @param  integer $post_id Post ID.
		 * @return void
		 */
		public function delete_imported_posts( $post_id = 0 ) {

			if ( gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$post_id = isset( $_REQUEST['post_id'] ) ? absint( $_REQUEST['post_id'] ) : $post_id;

			$message = 'Deleted - Post ID ' . $post_id . ' - ' . get_post_type( $post_id ) . ' - ' . get_the_title( $post_id );

			$message = '';
			if ( $post_id ) {

				$post_type = get_post_type( $post_id );
				$message   = 'Deleted - Post ID ' . $post_id . ' - ' . $post_type . ' - ' . get_the_title( $post_id );

				do_action( 'adaptive_sites_before_delete_imported_posts', $post_id, $post_type );

				Adaptive_Sites_Importer_Log::add( $message );
				gc_delete_post( $post_id, true );
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( $message );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success( $message );
			}
		}

		/**
		 * Delete imported GC forms
		 *
		 * @since 1.3.0
		 * @since 1.4.0 The `$post_id` was added.
		 *
		 * @param  integer $post_id Post ID.
		 * @return void
		 */
		public function delete_imported_gc_forms( $post_id = 0 ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$post_id = isset( $_REQUEST['post_id'] ) ? absint( $_REQUEST['post_id'] ) : $post_id;

			$message = '';
			if ( $post_id ) {

				do_action( 'adaptive_sites_before_delete_imported_gc_forms', $post_id );

				$message = 'Deleted - Form ID ' . $post_id . ' - ' . get_post_type( $post_id ) . ' - ' . get_the_title( $post_id );
				Adaptive_Sites_Importer_Log::add( $message );
				gc_delete_post( $post_id, true );
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( $message );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success( $message );
			}
		}

		/**
		 * Delete imported terms
		 *
		 * @since 1.3.0
		 * @since 1.4.0 The `$post_id` was added.
		 *
		 * @param  integer $term_id Term ID.
		 * @return void
		 */
		public function delete_imported_terms( $term_id = 0 ) {
			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				// Verify Nonce.
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'customize' ) ) {
					gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
				}
			}

			$term_id = isset( $_REQUEST['term_id'] ) ? absint( $_REQUEST['term_id'] ) : $term_id;

			$message = '';
			if ( $term_id ) {
				$term = get_term( $term_id );
				if ( ! is_gc_error( $term ) ) {

					do_action( 'adaptive_sites_before_delete_imported_terms', $term_id, $term );

					$message = 'Deleted - Term ' . $term_id . ' - ' . $term->name . ' ' . $term->taxonomy;
					Adaptive_Sites_Importer_Log::add( $message );
					gc_delete_term( $term_id, $term->taxonomy );
				}
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( $message );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success( $message );
			}
		}

	}

	/**
	 * Kicking this off by calling 'get_instance()' method
	 */
	Adaptive_Sites_Importer::get_instance();
}
