<?php
/**
 * Adaptive Sites
 *
 * @since  1.0.0
 * @package Adaptive Sites
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Elementor\Core\Schemes;

if ( ! class_exists( 'Adaptive_Sites' ) ) :

	/**
	 * Adaptive_Sites
	 */
	class Adaptive_Sites {

		/**
		 * API Domain name
		 *
		 * @var (String) URL
		 */
		public $api_domain;

		/**
		 * API URL which is used to get the response from.
		 *
		 * @since  1.0.0
		 * @var (String) URL
		 */
		public $api_url;

		/**
		 * Search API URL which is used to get the response from.
		 *
		 * @since  2.0.0
		 * @var (String) URL
		 */
		public $search_url;

		/**
		 * API URL which is used to get the response from Pixabay.
		 *
		 * @since  2.0.0
		 * @var (String) URL
		 */
		public $pixabay_url;

		/**
		 * API Key which is used to get the response from Pixabay.
		 *
		 * @since  2.0.0
		 * @var (String) URL
		 */
		public $pixabay_api_key;

		/**
		 * Instance of Adaptive_Sites
		 *
		 * @since  1.0.0
		 * @var (Object) Adaptive_Sites
		 */
		private static $instance = null;

		/**
		 * Localization variable
		 *
		 * @since  2.0.0
		 * @var (Array) $local_vars
		 */
		public static $local_vars = array();

		/**
		 * Localization variable
		 *
		 * @since  2.0.0
		 * @var (Array) $gc_upload_url
		 */
		public $gc_upload_url = '';

		/**
		 * Instance of Adaptive_Sites.
		 *
		 * @since  1.0.0
		 *
		 * @return object Class object.
		 */
		public static function get_instance() {
			if ( ! isset( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * Constructor.
		 *
		 * @since  1.0.0
		 */
		private function __construct() {

			$this->set_api_url();

			$this->includes();

			add_filter( 'adaptive_sites_localize_vars', array( $this, 'update_vars' ) );
			add_filter( 'adaptive_sites_render_localize_vars', array( $this, 'update_vars' ) );
			add_filter( 'adaptive_sites_api_params', array( $this, 'api_request_params' ) );
			add_filter( 'adaptive_sites_menu_page_title', array( $this, 'page_title' ) );

			add_action( 'plugin_action_links_' . ADAPTIVE_SITES_BASE, array( $this, 'action_links' ) );
			add_action( 'plugins_loaded', array( $this, 'load_textdomain' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue' ), 99 );
			add_action( 'gc_enqueue_scripts', array( $this, 'image_search_scripts' ) );
			add_action( 'elementor/editor/footer', array( $this, 'insert_templates' ) );
			add_action( 'admin_footer', array( $this, 'insert_image_templates' ) );
			add_action( 'customize_controls_print_footer_scripts', array( $this, 'insert_image_templates' ) );
			add_action( 'gc_footer', array( $this, 'insert_image_templates_bb_and_brizy' ) );
			add_action( 'elementor/editor/footer', array( $this, 'register_widget_scripts' ), 99 );
			add_action( 'elementor/editor/before_enqueue_scripts', array( $this, 'popup_styles' ) );
			add_action( 'elementor/preview/enqueue_styles', array( $this, 'popup_styles' ) );

			// AJAX.
			add_action( 'gc_ajax_adaptive-required-plugins', array( $this, 'required_plugin' ) );
			add_action( 'gc_ajax_adaptive-required-plugin-activate', array( $this, 'required_plugin_activate' ) );

			add_action( 'gc_ajax_adaptive-required-themes', array( $this, 'required_theme' ) );
			add_action( 'gc_ajax_adaptive-required-theme-activate', array( $this, 'required_theme_activate' ) );

			add_action( 'gc_ajax_adaptive-sites-backup-settings', array( $this, 'backup_settings' ) );
			add_action( 'gc_ajax_adaptive-sites-set-reset-data', array( $this, 'get_reset_data' ) );
			// add_action( 'gc_ajax_adaptive-sites-activate-theme', array( $this, 'activate_theme' ) );
			add_action( 'gc_ajax_adaptive-sites-create-page', array( $this, 'create_page' ) );
			add_action( 'gc_ajax_adaptive-sites-import-media', array( $this, 'import_media' ) );
			add_action( 'gc_ajax_adaptive-sites-create-template', array( $this, 'create_template' ) );
			add_action( 'gc_ajax_adaptive-sites-create-image', array( $this, 'create_image' ) );
			add_action( 'gc_ajax_adaptive-sites-getting-started-notice', array( $this, 'getting_started_notice' ) );
			add_action( 'gc_ajax_adaptive-sites-favorite', array( $this, 'add_to_favorite' ) );
			add_action( 'gc_ajax_adaptive-sites-api-request', array( $this, 'api_request' ) );
			add_action( 'gc_ajax_adaptive-page-elementor-batch-process', array( $this, 'elementor_batch_process' ) );

			add_action( 'delete_attachment', array( $this, 'delete_adaptive_images' ) );
			add_filter( 'heartbeat_received', array( $this, 'search_push' ), 10, 2 );
			add_action( 'gc_ajax_adaptive-sites-update-subscription', array( $this, 'update_subscription' ) );
			add_action( 'admin_footer', array( $this, 'add_quick_links' ) );

		}

		/**
		 * Add BSF Quick Links.
		 */
		public function add_quick_links() {
			$current_screen = get_current_screen();

			if ( 'appearance_page_starter-templates' !== $current_screen->id ) {
				return;
			}

			if ( Adaptive_Sites_White_Label::get_instance()->is_white_labeled() ) {
				return;
			}

			$data = apply_filters(
				'adaptive_sites_quick_links', array(
					'default_logo' => array(
						'title' => __( '查看快速链接', 'adaptive-sites' ),
						'url'   => ADAPTIVE_SITES_URI . 'inc/assets/images/quick-link-logo.svg',
					),
					'links'        => array(
						'upgrade' => array(
							'label'   => __( '升级到高级版', 'adaptive-sites' ),
							'icon'    => 'dashicons-star-filled',
							'url'     => 'https://www.gechiui.com/shop/pro/adaptive-sites/',
							'bgcolor' => '#ffa500',
						),
						'support' => array(
							'label' => __( '支持和文档', 'adaptive-sites' ),
							'icon'  => 'dashicons-book',
							'url'   => 'https://docs.gechiui.com/adaptive-category/starter-templates/',
						),
						'join-facebook' => array(
							'label' => __( '加入Facebook群组', 'adaptive-sites' ),
							'icon'  => 'dashicons-groups',
							'url'   => 'https://www.facebook.com/groups/gcadaptive/',
						),
					),
				)
			);

			if ( defined( 'ADAPTIVE_SITES_VER' ) ) {
				unset( $data['links']['upgrade'] );
			}
		}

		/**
		 * Update Subscription
		 */
		public function update_subscription() {

			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'manage_options' ) ) {
				gc_send_json_error( 'You can\'t access this action.' );
			}

			$arguments = isset( $_POST['data'] ) ? array_map( 'sanitize_text_field', json_decode( stripslashes( $_POST['data'] ), true ) ) : array();

			// Page Builder mapping.
			$page_builder_mapping      = array(
				'Elementor'      => 1,
				'Gutenberg'      => 4,
			);
			$arguments['PAGE_BUILDER'] = isset( $page_builder_mapping[ $arguments['PAGE_BUILDER'] ] ) ? $page_builder_mapping[ $arguments['PAGE_BUILDER'] ] : '';

			$url = apply_filters( 'adaptive_sites_subscription_url', $this->api_domain . 'gc-json/starter-templates/v1/subscribe/' );

			$args = array(
				'timeout' => 30,
				'body'    => $arguments,
			);

			$response = gc_remote_post( $url, $args );

			if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
				$response = json_decode( gc_remote_retrieve_body( $response ), true );

				// Successfully subscribed.
				if ( isset( $response['success'] ) && $response['success'] ) {
					update_user_meta( get_current_user_ID(), 'adaptive-sites-subscribed', 'yes' );
				}
			}
			gc_send_json_success( $response );

		}

		/**
		 * Push Data to Search API.
		 *
		 * @since  2.0.0
		 * @param Object $response Response data object.
		 * @param Object $data Data object.
		 *
		 * @return array Search response.
		 */
		public function search_push( $response, $data ) {

			// If we didn't receive our data, don't send any back.
			if ( empty( $data['ast-sites-search-terms'] ) ) {
				return $response;
			}

			$args = array(
				'timeout'   => 3,
				'blocking'  => true,
				'sslverify' => apply_filters( 'https_local_ssl_verify', false ),
				'body'      => array(
					'search' => $data['ast-sites-search-terms'],
					'url'    => esc_url( site_url() ),
					'type'   => 'adaptive-sites',
				),
			);
			$result                             = gc_remote_post( $this->search_url, $args );
			$response['ast-sites-search-terms'] = gc_remote_retrieve_body( $result );

			return $response;
		}

		/**
		 * Before Adaptive Image delete, remove from options.
		 *
		 * @since  2.0.0
		 * @param int $id ID to deleting image.
		 * @return void
		 */
		public function delete_adaptive_images( $id ) {

			if ( ! $id ) {
				return;
			}

			// @codingStandardsIgnoreStart
			$saved_images     = get_option( 'adaptive-sites-saved-images', array() );
			$adaptive_image_flag = get_post_meta( $id, 'adaptive-images', true );
			$adaptive_image_flag = (int) $adaptive_image_flag;
			if (
				'' !== $adaptive_image_flag &&
				is_array( $saved_images ) &&
				! empty( $saved_images ) &&
				in_array( $adaptive_image_flag, $saved_images )
			) {
				$flag_arr = array( $adaptive_image_flag );
				$saved_images = array_diff( $saved_images, $flag_arr );
				update_option( 'adaptive-sites-saved-images', $saved_images, 'no' );
			}
			// @codingStandardsIgnoreEnd
		}

		/**
		 * Enqueue Image Search scripts into Beaver Builder Editor.
		 *
		 * @since  2.0.0
		 * @return void
		 */
		public function image_search_scripts() {

			if (
				class_exists( 'FLBuilderModel' ) && FLBuilderModel::is_builder_active() // BB Builder is on?
				||
				(
					class_exists( 'Brizy_Editor_Post' ) && // Brizy Builder is on?
					( isset( $_GET['brizy-edit'] ) || isset( $_GET['brizy-edit-iframe'] ) ) // phpcs:ignore GeChiUI.Security.NonceVerification.Recommended
				)
				||
				is_customize_preview() // Is customizer on?
			) {
				// Image Search assets.
				$this->image_search_assets();
			}
		}

		/**
		 * Elementor Batch Process via AJAX
		 *
		 * @since 2.0.0
		 */
		public function elementor_batch_process() {

			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'edit_posts' ) ) {
				gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
			}

			if ( ! isset( $_POST['url'] ) ) {
				gc_send_json_error( __( '无效的API URL', 'adaptive-sites' ) );
			}

			$response = gc_remote_get( $_POST['url'] );

			if ( is_gc_error( $response ) ) {
				gc_send_json_error( gc_remote_retrieve_body( $response ) );
			}

			$body = gc_remote_retrieve_body( $response );
			$data = json_decode( $body, true );
			if ( ! isset( $data['post-meta']['_elementor_data'] ) ) {
				gc_send_json_error( __( '无效的文章标签', 'adaptive-sites' ) );
			}

			$meta    = json_decode( $data['post-meta']['_elementor_data'], true );
			$post_id = $_POST['id'];

			if ( empty( $post_id ) || empty( $meta ) ) {
				gc_send_json_error( __( '无效的文章ID 或元素构建器标签', 'adaptive-sites' ) );
			}

			if ( isset( $data['adaptive-page-options-data'] ) && isset( $data['adaptive-page-options-data']['elementor_load_fa4_shim'] ) ) {
				update_option( 'elementor_load_fa4_shim', $data['adaptive-page-options-data']['elementor_load_fa4_shim'] );
			}

			$import      = new \Elementor\TemplateLibrary\Adaptive_Sites_Elementor_Pages();
			$import_data = $import->import( $post_id, $meta );

			gc_send_json_success( $import_data );
		}

		/**
		 * API Request
		 *
		 * @since 2.0.0
		 */
		public function api_request() {
			$url = isset( $_POST['url'] ) ? $_POST['url'] : ''; // phpcs:ignore GeChiUI.Security.NonceVerification.Missing

			if ( empty( $url ) ) {
				gc_send_json_error( __( '提供的API URL是空的!请再试一次!', 'adaptive-sites' ) );
			}

			$api_args = apply_filters(
				'adaptive_sites_api_args',
				array(
					'timeout' => 30,
				)
			);

			$request = gc_remote_get( trailingslashit( self::get_instance()->get_api_domain() ) . 'gc-json/gc/v2/' . $url, $api_args );

			if ( is_gc_error( $request ) ) {
				$gc_error_code = $request->get_error_code();
				switch ( $gc_error_code ) {
					case 'http_request_not_executed':
						/* translators: %s Error Message */
						$message = sprintf( __( '无法执行 API 请求 - %s', 'adaptive-sites' ), $request->get_error_message() );
						break;
					case 'http_request_failed':
					default:
						/* translators: %s Error Message */
						$message = sprintf( __( 'API 请求已失败 - %s', 'adaptive-sites' ), $request->get_error_message() );
						break;
				}

				gc_send_json_error(
					array(
						'message'       => $request->get_error_message(),
						'code'          => 'GC_Error',
						'response_code' => $gc_error_code,
					)
				);
			}

			$code      = (int) gc_remote_retrieve_response_code( $request );
			$demo_data = json_decode( gc_remote_retrieve_body( $request ), true );

			if ( 200 === $code ) {
				update_option( 'adaptive_sites_import_data', $demo_data, 'no' );
				gc_send_json_success( $demo_data );
			}

			$message       = gc_remote_retrieve_body( $request );
			$response_code = $code;

			if ( 200 !== $code && is_array( $demo_data ) && isset( $demo_data['code'] ) ) {
				$message = $demo_data['message'];
			}

			if ( 500 === $code ) {
				$message = __( '内部服务器错误。', 'adaptive-sites' );
			}

			if ( 200 !== $code && false !== strpos( $message, 'Cloudflare' ) ) {
				$ip = Adaptive_Sites_Helper::get_client_ip();
				/* translators: %s IP address. */
				$message = sprintf( __( '客户 IP： %1$s </br> 错误代码： %2$s', 'adaptive-sites' ), $ip, $code );
				$code    = 'Cloudflare';
			}

			gc_send_json_error(
				array(
					'message'       => $message,
					'code'          => $code,
					'response_code' => $response_code,
				)
			);
		}

		/**
		 * Insert Template
		 *
		 * @return void
		 */
		public function insert_image_templates() {
			ob_start();
			require_once ADAPTIVE_SITES_DIR . 'inc/includes/image-templates.php';
			ob_end_flush();
		}

		/**
		 * Insert Template
		 *
		 * @return void
		 */
		public function insert_image_templates_bb_and_brizy() {

			if (
				class_exists( 'FLBuilderModel' ) && FLBuilderModel::is_builder_active() // BB Builder is on?
				||
				(
					class_exists( 'Brizy_Editor_Post' ) && // Brizy Builder is on?
					( isset( $_GET['brizy-edit'] ) || isset( $_GET['brizy-edit-iframe'] ) ) // phpcs:ignore GeChiUI.Security.NonceVerification.Recommended
				)
			) {
				// Image Search Templates.
				ob_start();
				require_once ADAPTIVE_SITES_DIR . 'inc/includes/image-templates.php';
				ob_end_flush();
			}
		}

		/**
		 * Insert Template
		 *
		 * @return void
		 */
		public function insert_templates() {
			ob_start();
			require_once ADAPTIVE_SITES_DIR . 'inc/includes/templates.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/includes/image-templates.php';
			ob_end_flush();
		}

		/**
		 * Add/Remove Favorite.
		 *
		 * @since  2.0.0
		 */
		public function add_to_favorite() {

			if ( ! current_user_can( 'manage_options' ) ) {
				gc_send_json_error( 'You can\'t access this action.' );
			}

			$new_favorites = array();
			$site_id       = isset( $_POST['site_id'] ) ? sanitize_key( $_POST['site_id'] ) : ''; // phpcs:ignore GeChiUI.Security.NonceVerification.Missing

			if ( empty( $site_id ) ) {
				gc_send_json_error();
			}

			$favorite_settings = get_option( 'adaptive-sites-favorites', array() );

			if ( false !== $favorite_settings && is_array( $favorite_settings ) ) {
				$new_favorites = $favorite_settings;
			}

			if ( 'false' === $_POST['is_favorite'] ) { // phpcs:ignore GeChiUI.Security.NonceVerification.Missing
				if ( in_array( $site_id, $new_favorites, true ) ) {
					$key = array_search( $site_id, $new_favorites, true );
					unset( $new_favorites[ $key ] );
				}
			} else {
				if ( ! in_array( $site_id, $new_favorites, true ) ) {
					array_push( $new_favorites, $site_id );
				}
			}

			update_option( 'adaptive-sites-favorites', $new_favorites, 'no' );

			gc_send_json_success(
				array(
					'all_favorites' => $new_favorites,
				)
			);
		}

		/**
		 * Import Template.
		 *
		 * @since  2.0.0
		 */
		public function create_template() {

			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'customize' ) ) {
				gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
			}

			$content = isset( $_POST['data']['content']['rendered'] ) ? $_POST['data']['content']['rendered'] : '';

			$data = isset( $_POST['data'] ) ? $_POST['data'] : array();

			if ( empty( $data ) ) {
				gc_send_json_error( 'Empty page data.' );
			}

			$page_id = isset( $_POST['data']['id'] ) ? $_POST['data']['id'] : '';

			$title = '';
			if ( isset( $_POST['data']['title']['rendered'] ) ) {
				if ( '' !== $_POST['title'] ) {
					$title = $_POST['title'] . ' - ' . $_POST['data']['title']['rendered'];
				} else {
					$title = $_POST['data']['title']['rendered'];
				}
			}

			$excerpt = isset( $_POST['data']['excerpt']['rendered'] ) ? $_POST['data']['excerpt']['rendered'] : '';

			$post_args = array(
				'post_type'    => 'elementor_library',
				'post_status'  => 'publish',
				'post_title'   => $title,
				'post_content' => $content,
				'post_excerpt' => $excerpt,
			);

			$new_page_id = gc_insert_post( $post_args );
			update_post_meta( $new_page_id, '_adaptive_sites_enable_for_batch', true );
			$post_meta = isset( $_POST['data']['post-meta'] ) ? $_POST['data']['post-meta'] : array();

			if ( ! empty( $post_meta ) ) {
				$this->import_template_meta( $new_page_id, $post_meta );
			}

			if ( 'pages' === $_POST['type'] ) {
				update_post_meta( $new_page_id, '_elementor_template_type', 'page' );
				gc_set_object_terms( $new_page_id, 'page', 'elementor_library_type' );
			} else {
				update_post_meta( $new_page_id, '_elementor_template_type', 'section' );
				gc_set_object_terms( $new_page_id, 'section', 'elementor_library_type' );
			}

			update_post_meta( $new_page_id, '_gc_page_template', 'elementor_header_footer' );

			do_action( 'adaptive_sites_process_single', $new_page_id );

			gc_send_json_success(
				array(
					'remove-page-id' => $page_id,
					'id'             => $new_page_id,
					'link'           => get_permalink( $new_page_id ),
				)
			);
		}

		/**
		 * Import Birzy Media.
		 *
		 * @since  2.0.0
		 */
		public function import_media() {

			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			$image_data = isset( $_POST['media'] ) ? $_POST['media'] : array();

			if ( empty( $image_data ) ) {
				gc_send_json_error();
			}

			$image            = array(
				'url' => $image_data['url'],
				'id'  => $image_data['id'],
			);
			$downloaded_image = Adaptive_Sites_Image_Importer::get_instance()->import( $image );

			// Set meta data.
			if ( isset( $image_data['meta'] ) && ! empty( $image_data['meta'] ) ) {
				foreach ( $image_data['meta'] as $meta_key => $meta_value ) {
					update_post_meta( $downloaded_image['id'], $meta_key, $meta_value );
				}
			}

			gc_send_json_success();
		}

		/**
		 * Import Page.
		 *
		 * @since  2.0.0
		 */
		public function create_page() {

			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'customize' ) ) {
				gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
			}

			$default_page_builder = Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' );

			$content = isset( $_POST['data']['original_content'] ) ? $_POST['data']['original_content'] : ( isset( $_POST['data']['content']['rendered'] ) ? $_POST['data']['content']['rendered'] : '' );

			if ( 'elementor' === $default_page_builder ) {
				if ( isset( $_POST['data']['adaptive-page-options-data'] ) && isset( $_POST['data']['adaptive-page-options-data']['elementor_load_fa4_shim'] ) ) {
					update_option( 'elementor_load_fa4_shim', $_POST['data']['adaptive-page-options-data']['elementor_load_fa4_shim'] );
				}
			}

			$data = isset( $_POST['data'] ) ? $_POST['data'] : array();

			if ( empty( $data ) ) {
				gc_send_json_error( 'Empty page data.' );
			}

			$page_id = isset( $_POST['data']['id'] ) ? $_POST['data']['id'] : '';
			$title   = isset( $_POST['data']['title']['rendered'] ) ? $_POST['data']['title']['rendered'] : '';
			$excerpt = isset( $_POST['data']['excerpt']['rendered'] ) ? $_POST['data']['excerpt']['rendered'] : '';

			$post_args = array(
				'post_type'    => 'page',
				'post_status'  => 'draft',
				'post_title'   => $title,
				'post_content' => $content,
				'post_excerpt' => $excerpt,
			);

			$new_page_id = gc_insert_post( $post_args );
			update_post_meta( $new_page_id, '_adaptive_sites_enable_for_batch', true );

			$post_meta = isset( $_POST['data']['post-meta'] ) ? $_POST['data']['post-meta'] : array();

			if ( ! empty( $post_meta ) ) {
				$this->import_post_meta( $new_page_id, $post_meta );
			}

			if ( isset( $_POST['data']['adaptive-page-options-data'] ) && ! empty( $_POST['data']['adaptive-page-options-data'] ) ) {

				foreach ( $_POST['data']['adaptive-page-options-data'] as $option => $value ) {
					update_option( $option, $value );
				}
			}

			if ( 'elementor' === $default_page_builder ) {
				update_post_meta( $new_page_id, '_gc_page_template', 'elementor_header_footer' );
			}

			do_action( 'adaptive_sites_process_single', $new_page_id );

			gc_send_json_success(
				array(
					'remove-page-id' => $page_id,
					'id'             => $new_page_id,
					'link'           => get_permalink( $new_page_id ),
				)
			);
		}

		/**
		 * Download and save the image in the media library.
		 *
		 * @since  2.0.0
		 */
		public function create_image() {
			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'edit_posts' ) ) {
				gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
			}

			$url      = isset( $_POST['url'] ) ? esc_url_raw( $_POST['url'] ) : false;
			$name     = isset( $_POST['name'] ) ? sanitize_text_field( $_POST['name'] ) : false;
			$photo_id = isset( $_POST['id'] ) ? absint( $_POST['id'] ) : 0;

			if ( false === $url ) {
				gc_send_json_error( __( '需要发送要下载的图像的 URL', 'adaptive-sites' ) );
			}

			$image  = '';
			$result = array();

			$name  = preg_replace( '/\.[^.]+$/', '', $name ) . '-' . $photo_id . '.jpg';
			$image = $this->create_image_from_url( $url, $name, $photo_id );

			if ( is_gc_error( $image ) ) {
				gc_send_json_error( $image );
			}

			if ( 0 !== $image ) {
				$result['attachmentData'] = gc_prepare_attachment_for_js( $image );
				if ( did_action( 'elementor/loaded' ) ) {
					$result['data'] = Adaptive_Sites_Elementor_Images::get_instance()->get_attachment_data( $image );
				}
				if ( 0 === $photo_id ) {
					/**
					 * This flag ensures these files are deleted in the Reset Process.
					 */
					update_post_meta( $image, '_adaptive_sites_imported_post', true );
				}
			} else {
				gc_send_json_error( __( '无法下载图像。', 'adaptive-sites' ) );
			}

			// Save downloaded image reference to an option.
			if ( 0 !== $photo_id ) {
				$saved_images = get_option( 'adaptive-sites-saved-images', array() );

				if ( empty( $saved_images ) || false === $saved_images ) {
					$saved_images = array();
				}

				$saved_images[] = $photo_id;
				update_option( 'adaptive-sites-saved-images', $saved_images, 'no' );
			}

			$result['updated-saved-images'] = get_option( 'adaptive-sites-saved-images', array() );

			gc_send_json_success( $result );
		}

		/**
		 * Set the upload directory
		 */
		public function get_gc_upload_url() {
			$gc_upload_dir = gc_upload_dir();
			return isset( $gc_upload_dir['url'] ) ? $gc_upload_dir['url'] : false;
		}

		/**
		 * Create the image and return the new media upload id.
		 *
		 * @param String $url URL to pixabay image.
		 * @param String $name Name to pixabay image.
		 * @param String $photo_id Photo ID to pixabay image.
		 * @see http://codex.gechiui.com/Function_Reference/gc_insert_attachment#Example
		 */
		public function create_image_from_url( $url, $name, $photo_id ) {
			$file_array         = array();
			$file_array['name'] = gc_basename( $name );

			// Download file to temp location.
			$file_array['tmp_name'] = download_url( $url );

			// If error storing temporarily, return the error.
			if ( is_gc_error( $file_array['tmp_name'] ) ) {
				return $file_array;
			}

			// Do the validation and storage stuff.
			$id = media_handle_sideload( $file_array, 0, null );

			// If error storing permanently, unlink.
			if ( is_gc_error( $id ) ) {
				@unlink( $file_array['tmp_name'] ); // phpcs:ignore GeChiUI.PHP.NoSilencedErrors.Discouraged
				return $id;
			}

			// Store the original attachment source in meta.
			add_post_meta( $id, '_source_url', $url );

			update_post_meta( $id, 'adaptive-images', $photo_id );
			update_post_meta( $id, '_gc_attachment_image_alt', $name );

			return $id;
		}

		/**
		 * Import Post Meta
		 *
		 * @since 2.0.0
		 *
		 * @param  integer $post_id  Post ID.
		 * @param  array   $metadata  Post meta.
		 * @return void
		 */
		public function import_post_meta( $post_id, $metadata ) {

			$metadata = (array) $metadata;

			foreach ( $metadata as $meta_key => $meta_value ) {

				if ( $meta_value ) {

					if ( '_elementor_data' === $meta_key ) {

						$raw_data = json_decode( stripslashes( $meta_value ), true );

						if ( is_array( $raw_data ) ) {
							$raw_data = gc_slash( gc_json_encode( $raw_data ) );
						} else {
							$raw_data = gc_slash( $raw_data );
						}
					} else {

						if ( is_serialized( $meta_value, true ) ) {
							$raw_data = maybe_unserialize( stripslashes( $meta_value ) );
						} elseif ( is_array( $meta_value ) ) {
							$raw_data = json_decode( stripslashes( $meta_value ), true );
						} else {
							$raw_data = $meta_value;
						}
					}

					update_post_meta( $post_id, $meta_key, $raw_data );
				}
			}
		}

		/**
		 * Import Post Meta
		 *
		 * @since 2.0.0
		 *
		 * @param  integer $post_id  Post ID.
		 * @param  array   $metadata  Post meta.
		 * @return void
		 */
		public function import_template_meta( $post_id, $metadata ) {

			$metadata = (array) $metadata;

			foreach ( $metadata as $meta_key => $meta_value ) {

				if ( $meta_value ) {

					if ( '_elementor_data' === $meta_key ) {

						$raw_data = json_decode( stripslashes( $meta_value ), true );

						if ( is_array( $raw_data ) ) {
							$raw_data = gc_slash( gc_json_encode( $raw_data ) );
						} else {
							$raw_data = gc_slash( $raw_data );
						}
					} else {

						if ( is_serialized( $meta_value, true ) ) {
							$raw_data = maybe_unserialize( stripslashes( $meta_value ) );
						} elseif ( is_array( $meta_value ) ) {
							$raw_data = json_decode( stripslashes( $meta_value ), true );
						} else {
							$raw_data = $meta_value;
						}
					}

					update_post_meta( $post_id, $meta_key, $raw_data );
				}
			}
		}

		/**
		 * Close getting started notice for current user
		 *
		 * @since 1.3.5
		 * @return void
		 */
		public function getting_started_notice() {
			// Verify Nonce.
			check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

			if ( ! current_user_can( 'customize' ) ) {
				gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
			}

			update_option( '_adaptive_sites_gettings_started', 'yes', 'no' );
			gc_send_json_success();
		}

		/**
		 * Activate theme
		 *
		 * @since 1.3.2
		 * @return void
		 */
		// public function activate_theme() {

		// 	// Verify Nonce.
		// 	check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

		// 	if ( ! current_user_can( 'customize' ) ) {
		// 		gc_send_json_error( __( '您不允许执行此操作', 'adaptive-sites' ) );
		// 	}

		// 	$demo_data = get_option( 'adaptive_sites_import_data', array() );
		// 	if ( isset( $demo_data['required-themes'][0] ) ) {
		// 		switch_theme( $demo_data['required-themes'][0]['slug'] );
		// 	}else{
		// 		switch_theme( 'adaptive' );
		// 	}

		// 	gc_send_json_success(
		// 		array(
		// 			'success' => true,
		// 			'message' => __( '主题激活', 'adaptive-sites' ),
		// 		)
		// 	);
		// }

		/**
		 * Set reset data
		 */
		public function get_reset_data() {

			if ( gc_doing_ajax() ) {
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'manage_options' ) ) {
					return;
				}
			}

			global $gcdb;

			$post_ids = $gcdb->get_col( "SELECT post_id FROM {$gcdb->postmeta} WHERE meta_key='_adaptive_sites_imported_post'" );
			$form_ids = $gcdb->get_col( "SELECT post_id FROM {$gcdb->postmeta} WHERE meta_key='_adaptive_sites_imported_gc_forms'" );
			$term_ids = $gcdb->get_col( "SELECT term_id FROM {$gcdb->termmeta} WHERE meta_key='_adaptive_sites_imported_term'" );

			$data = array(
				'reset_posts'    => $post_ids,
				'reset_gc_forms' => $form_ids,
				'reset_terms'    => $term_ids,
			);

			if ( gc_doing_ajax() ) {
				gc_send_json_success( $data );
			}

			return $data;
		}

		/**
		 * Backup our existing settings.
		 */
		public function backup_settings() {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'manage_options' ) ) {
					gc_send_json_error( __( '用户没有权限!', 'adaptive-sites' ) );
				}
			}

			$file_name    = 'adaptive-sites-backup-' . gmdate( 'd-M-Y-h-i-s' ) . '.json';
			$old_settings = get_option( 'adaptive-settings', array() );
			$upload_dir   = Adaptive_Sites_Importer_Log::get_instance()->log_dir();
			$upload_path  = trailingslashit( $upload_dir['path'] );
			$log_file     = $upload_path . $file_name;
			$file_system  = self::get_instance()->get_filesystem();

			// If file system fails? Then take a backup in site option.
			if ( false === $file_system->put_contents( $log_file, gc_json_encode( $old_settings ), FS_CHMOD_FILE ) ) {
				update_option( 'adaptive_sites_' . $file_name, $old_settings, 'no' );
			}

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( 'File generated at ' . $log_file );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success();
			}
		}

		/**
		 * 获取主题安装、是否活动的状态。
		 *
		 * @since 1.3.2
		 *
		 * @return string Theme status
		 */
		public function get_theme_status() {
			// 当前站点主题使用情况
			$theme = gc_get_theme();
			// 想安装站点模板的数据
			$demo_data = get_option( 'adaptive_sites_import_data', array() );

			// Theme installed and activate.
			if ( isset( $demo_data['required-themes'][0] ) ) {
				if( $demo_data['required-themes'][0]['slug'] == $theme->Template ){
					return 'installed-and-active';
				}
				// Theme installed but not activate.
				foreach ( (array) gc_get_themes() as $theme_dir => $theme ) {
					if ( $demo_data['required-themes'][0]['slug'] === $theme->Template ) {
						return 'installed-but-inactive';
					}
				}
			}

			return 'not-installed';
		}

		/**
		 * API Request Params
		 *
		 * @since 1.0.5
		 *
		 * @param  array $args API request arguments.
		 * @return arrray       Filtered API request params.
		 */
		public function api_request_params( $args = array() ) {

			$args['site_url']     = site_url();
			$args['purchase_key'] = Adaptive_Sites::get_instance()->get_license_key();

			return $args;
		}

		/**
		 * Page Title
		 *
		 * @since 1.0.0
		 *
		 * @param  string $title Page Title.
		 * @return string        Filtered Page Title.
		 */
		public function page_title( $title = '' ) {
			return Adaptive_Sites_White_Label::get_option( 'adaptive-sites', 'name', ADAPTIVE_SITES_NAME );
		}

		/**
		 * Update Vars
		 *
		 * @since 1.0.0
		 *
		 * @param  array $vars Localize variables.
		 * @return array        Filtered localize variables.
		 */
		public function update_vars( $vars = array() ) {

			$vars['getProText'] = __( '获取！', 'adaptive-sites' );
			$vars['getProURL']  = admin_url( 'plugins.php?bsf-inline-license-form=adaptive-sites' );

			return $vars;
		}

		/**
		 * Loads textdomain for the plugin.
		 *
		 * @since 1.0.1
		 */
		public function load_textdomain() {
			load_plugin_textdomain( 'adaptive-sites' );
		}


		/**
		 * Plugin Deactivation Link
		 *
		 * @since 1.0.0
		 *
		 * @param  string $slug Plugin Slug.
		 * @return string       Plugin Deactivation Link.
		 */
		private function deactivation_link( $slug = 'adaptive-sites' ) {

			$deactivate_url = admin_url( 'plugins.php' );
			if ( is_plugin_active_for_network( ADAPTIVE_SITES_BASE ) ) {
				$deactivate_url = network_admin_url( 'plugins.php' );
			}
			return add_query_arg(
				array(
					'action'        => 'deactivate',
					'plugin'        => rawurlencode( $slug . '/' . $slug . '.php' ),
					'plugin_status' => 'all',
					'paged'         => '1',
					'_gcnonce'      => gc_create_nonce( 'deactivate-plugin_' . $slug . '/' . $slug . '.php' ),
				),
				$deactivate_url
			);
		}

		/**
		 * Show action links on the plugin screen.
		 *
		 * @param   mixed $links Plugin Action links.
		 * @return  array
		 */
		public function action_links( $links = array() ) {

			$arguments = array(
				'page' => 'starter-templates',
			);

			$current_page_builder = Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' );
			if ( empty( $current_page_builder ) ) {
				$arguments['change-page-builder'] = 'yes';
			}
			$url = add_query_arg( $arguments, admin_url( 'themes.php' ) );

			$action_links = array(
				'settings' => '<a href="' . esc_url( $url ) . '" aria-label="' . esc_attr__( '查看模板', 'adaptive-sites' ) . '">' . esc_html__( '查看模板', 'adaptive-sites' ) . '</a>',
			);

			return array_merge( $action_links, $links );
		}

		/**
		 * Get the API URL.
		 *
		 * @since  1.0.0
		 */
		public static function get_api_domain() {
			return defined( 'STARTER_TEMPLATES_REMOTE_URL' ) ? STARTER_TEMPLATES_REMOTE_URL : apply_filters( 'adaptive_sites_api_domain', 'https://sitedemos.gechiui.com/' );
		}

		/**
		 * Setter for $api_url
		 *
		 * @since  1.0.0
		 */
		public function set_api_url() {
			$this->api_domain = trailingslashit( self::get_api_domain() );
			$this->api_url    = apply_filters( 'adaptive_sites_api_url', $this->api_domain . 'gc-json/gc/v2/' );

			$this->search_url = apply_filters( 'adaptive_sites_search_api_url', $this->api_domain . 'gc-json/analytics/v2/search/' );

			$this->pixabay_url     = 'https://pixabay.com/api/';
			$this->pixabay_api_key = '2727911-c4d7c1031949c7e0411d7e81e';
		}

		/**
		 * Enqueue Image Search scripts.
		 *
		 * @since  2.0.0
		 * @return void
		 */
		public function image_search_assets() {

			gc_enqueue_script( 'masonry' );
			gc_enqueue_script( 'imagesloaded' );

			gc_enqueue_script(
				'adaptive-sites-images-common',
				ADAPTIVE_SITES_URI . 'inc/assets/js/common.js',
				array( 'jquery', 'gc-util' ), // Dependencies, defined above.
				ADAPTIVE_SITES_VER,
				true
			);

			$data = apply_filters(
				'adaptive_sites_images_common',
				array(
					'ajaxurl'             => esc_url( admin_url( 'admin-ajax.php' ) ),
					'asyncurl'            => esc_url( admin_url( 'async-upload.php' ) ),
					'pixabay_url'         => $this->pixabay_url,
					'pixabay_api_key'     => $this->pixabay_api_key,
					'is_bb_active'        => ( class_exists( 'FLBuilderModel' ) ),
					'is_brizy_active'     => ( class_exists( 'Brizy_Editor_Post' ) ),
					'is_elementor_active' => ( did_action( 'elementor/loaded' ) ),
					'is_elementor_editor' => ( did_action( 'elementor/loaded' ) ) ? Elementor\Plugin::instance()->editor->is_edit_mode() : false,
					'is_bb_editor'        => ( class_exists( 'FLBuilderModel' ) ) ? ( FLBuilderModel::is_builder_active() ) : false,
					'is_brizy_editor'     => ( class_exists( 'Brizy_Editor_Post' ) ) ? ( isset( $_GET['brizy-edit'] ) || isset( $_GET['brizy-edit-iframe'] ) ) : false, // phpcs:ignore GeChiUI.Security.NonceVerification.Recommended
					'saved_images'        => get_option( 'adaptive-sites-saved-images', array() ),
					'pixabay_category'    => array(
						'all'            => __( '所有', 'adaptive-sites' ),
						'animals'        => __( '动物', 'adaptive-sites' ),
						'buildings'      => __( '建设/创建', 'adaptive-sites' ),
						'backgrounds'    => __( '背景/纹理', 'adaptive-sites' ),
						'fashion'        => __( '美容/时尚', 'adaptive-sites' ),
						'business'       => __( '业务/财务', 'adaptive-sites' ),
						'computer'       => __( '计算机/通信', 'adaptive-sites' ),
						'education'      => __( '教育', 'adaptive-sites' ),
						'feelings'       => __( '情感', 'adaptive-sites' ),
						'food'           => __( '食物/饮料', 'adaptive-sites' ),
						'health'         => __( '健康&医疗', 'adaptive-sites' ),
						'industry'       => __( '工业/工艺', 'adaptive-sites' ),
						'music'          => __( '音乐', 'adaptive-sites' ),
						'nature'         => __( '自然/风景', 'adaptive-sites' ),
						'people'         => __( '個人', 'adaptive-sites' ),
						'places'         => __( '地点/纪念品', 'adaptive-sites' ),
						'religion'       => __( '关系', 'adaptive-sites' ),
						'science'        => __( '科学/技术', 'adaptive-sites' ),
						'sports'         => __( '体育活动中心', 'adaptive-sites' ),
						'transportation' => __( '交通/交通工具', 'adaptive-sites' ),
						'travel'         => __( '旅行/度假', 'adaptive-sites' ),
					),
					'pixabay_order'       => array(
						'popular'  => __( '流行', 'adaptive-sites' ),
						'latest'   => __( '最新的', 'adaptive-sites' ),
						'upcoming' => __( '即将到来', 'adaptive-sites' ),
						'ec'       => __( '编辑选择', 'adaptive-sites' ),
					),
					'pixabay_orientation' => array(
						'any'        => __( '任何方向', 'adaptive-sites' ),
						'vertical'   => __( '竖直', 'adaptive-sites' ),
						'horizontal' => __( '水平', 'adaptive-sites' ),
					),
					'title'               => __( '免费图片', 'adaptive-sites' ),
					'search_placeholder'  => __( '搜索-例如：鲜花', 'adaptive-sites' ),
					'downloading'         => __( '下载中', 'adaptive-sites' ),
					'validating'          => __( '验证中...', 'adaptive-sites' ),
					'empty_api_key'       => __( '请输入API密钥。', 'adaptive-sites' ),
					'error_api_key'       => __( '代码发生错误 ', 'adaptive-sites' ),
					'_ajax_nonce'         => gc_create_nonce( 'adaptive-sites' ),
				)
			);
			gc_localize_script( 'adaptive-sites-images-common', 'adaptiveImages', $data );

			gc_enqueue_script(
				'adaptive-sites-images-script',
				ADAPTIVE_SITES_URI . 'inc/assets/js/dist/main.js',
				array( 'gc-blocks', 'gc-i18n', 'gc-element', 'gc-components', 'gc-api-fetch', 'adaptive-sites-images-common' ), // Dependencies, defined above.
				ADAPTIVE_SITES_VER,
				true
			);

			gc_enqueue_style( 'adaptive-sites-images', ADAPTIVE_SITES_URI . 'inc/assets/css/images.css', ADAPTIVE_SITES_VER, true );
			gc_style_add_data( 'adaptive-sites-images', 'rtl', 'replace' );
		}

		/**
		 * Getter for $api_url
		 *
		 * @since  1.0.0
		 */
		public function get_api_url() {
			return $this->api_url;
		}

		/**
		 * Enqueue admin scripts.
		 *
		 * @since  1.3.2    Added 'install-theme.js' to install and activate theme.
		 * @since  1.0.5    Added 'getUpgradeText' and 'getUpgradeURL' localize variables.
		 *
		 * @since  1.0.0
		 *
		 * @param  string $hook Current hook name.
		 * @return void
		 */
		public function admin_enqueue( $hook = '' ) {

			// Image Search assets.
			if ( 'post-new.php' === $hook || 'post.php' === $hook || 'widgets.php' === $hook ) {
				$this->image_search_assets();
			}

			// Avoid scripts from customizer.
			if ( is_customize_preview() ) {
				return;
			}

			gc_enqueue_script( 'adaptive-sites-install-theme', ADAPTIVE_SITES_URI . 'inc/assets/js/install-theme.js', array( 'jquery', 'updates' ), ADAPTIVE_SITES_VER, true );
			gc_enqueue_style( 'adaptive-sites-install-theme', ADAPTIVE_SITES_URI . 'inc/assets/css/install-theme.css', null, ADAPTIVE_SITES_VER, 'all' );
			gc_style_add_data( 'adaptive-sites-install-theme', 'rtl', 'replace' );

			$data = apply_filters(
				'adaptive_sites_install_theme_localize_vars',
				array(
					'installed'   => __( '已安装!激活中..', 'adaptive-sites' ),
					'activating'  => __( '正在启用...', 'adaptive-sites' ),
					'activated'   => __( '激活！', 'adaptive-sites' ),
					'installing'  => __( '正在安装…', 'adaptive-sites' ),
					'ajaxurl'     => esc_url( admin_url( 'admin-ajax.php' ) ),
					'_ajax_nonce' => gc_create_nonce( 'adaptive-sites' ),
				)
			);
			gc_localize_script( 'adaptive-sites-install-theme', 'AdaptiveSitesInstallThemeVars', $data );

			if ( 'appearance_page_starter-templates' !== $hook ) {
				return;
			}

			global $is_IE, $is_edge;

			if ( $is_IE || $is_edge ) {
				gc_enqueue_script( 'adaptive-sites-eventsource', ADAPTIVE_SITES_URI . 'inc/assets/js/eventsource.min.js', array( 'jquery', 'gc-util', 'updates' ), ADAPTIVE_SITES_VER, true );
			}

			// Fetch.
			gc_register_script( 'adaptive-sites-fetch', ADAPTIVE_SITES_URI . 'inc/assets/js/fetch.umd.js', array( 'jquery' ), ADAPTIVE_SITES_VER, true );

			// History.
			gc_register_script( 'adaptive-sites-history', ADAPTIVE_SITES_URI . 'inc/assets/js/history.js', array( 'jquery' ), ADAPTIVE_SITES_VER, true );

			// API.
			gc_register_script( 'adaptive-sites-api', ADAPTIVE_SITES_URI . 'inc/assets/js/adaptive-sites-api.js', array( 'jquery', 'adaptive-sites-fetch' ), ADAPTIVE_SITES_VER, true );

			// Admin Page.
			gc_enqueue_style( 'adaptive-sites-admin', ADAPTIVE_SITES_URI . 'inc/assets/css/admin.css', ADAPTIVE_SITES_VER, true );
			gc_style_add_data( 'adaptive-sites-admin', 'rtl', 'replace' );

			gc_enqueue_script( 'adaptive-sites-admin-page', ADAPTIVE_SITES_URI . 'inc/assets/js/admin-page.js', array( 'jquery', 'gc-util', 'updates', 'jquery-ui-autocomplete', 'adaptive-sites-api', 'adaptive-sites-history' ), ADAPTIVE_SITES_VER, true );

			$data = $this->get_local_vars();

			gc_localize_script( 'adaptive-sites-admin-page', 'adaptiveSitesVars', $data );
		}

		/**
		 * Get CTA link
		 *
		 * @param string $source    The source of the link.
		 * @param string $medium    The medium of the link.
		 * @param string $campaign  The campaign of the link.
		 * @return array
		 */
		public function get_cta_link( $source = '', $medium = '', $campaign = '' ) {
			$default_page_builder = Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' );
			$cta_links = $this->get_cta_links( $source, $medium, $campaign );
			return isset( $cta_links[ $default_page_builder ] ) ? $cta_links[ $default_page_builder ] : 'https://www.gechiui.com/shop/pro/adaptive-sites/';
		}

		/**
		 * Get CTA Links
		 *
		 * @since 2.6.18
		 *
		 * @param string $source    The source of the link.
		 * @param string $medium    The medium of the link.
		 * @param string $campaign  The campaign of the link.
		 * @return array
		 */
		public function get_cta_links( $source = '', $medium = '', $campaign = '' ) {
			return array(
				'elementor' => add_query_arg(
					array(
						'utm_source' => ! empty( $source ) ? $source : 'elementor-templates',
						'utm_medium' => 'dashboard',
						'utm_campaign' => 'Starter-Template-Backend',
					), 'https://www.gechiui.com/shop/pro/adaptive-sites/'
				),
				'gutenberg' => add_query_arg(
					array(
						'utm_source' => ! empty( $source ) ? $source : 'gutenberg-templates',
						'utm_medium' => 'dashboard',
						'utm_campaign' => 'Starter-Template-Backend',
					), 'https://www.gechiui.com/shop/pro/adaptive-sites/'
				),
			);
		}

		/**
		 * Returns Localization Variables.
		 *
		 * @since 2.0.0
		 */
		public function get_local_vars() {

			$stored_data = array(
				'adaptive-site-category'        => array(),
				'adaptive-site-page-builder'    => array(),
				'adaptive-sites'                => array(),
				'site-pages-category'        => array(),
				'site-pages-page-builder'    => array(),
				'site-pages-parent-category' => array(),
				'site-pages'                 => array(),
				'favorites'                  => get_option( 'adaptive-sites-favorites' ),
			);

			$favorite_data = get_option( 'adaptive-sites-favorites' );

			// Use this for premium demos.
			$request_params = apply_filters(
				'adaptive_sites_api_params',
				array(
					'purchase_key' => '',
					'site_url'     => '',
					'per-page'     => 15,
				)
			);

			$license_status = get_pro_license_valid();

			$default_page_builder = Adaptive_Sites_Page::get_instance()->get_setting( 'page_builder' );

			$data = apply_filters(
				'adaptive_sites_localize_vars',
				array(
					'subscribed'                         => get_user_meta( get_current_user_ID(), 'adaptive-sites-subscribed', true ),
					'debug'                              => defined( 'GC_DEBUG' ) ? true : false,
					'isPro'                              => defined( 'ADAPTIVE_SITES_NAME' ) ? true : false,
					'isWhiteLabeled'                     => Adaptive_Sites_White_Label::get_instance()->is_white_labeled(),
					'whiteLabelName'                     => Adaptive_Sites_White_Label::get_instance()->get_white_label_name(),
					'ajaxurl'                            => esc_url( admin_url( 'admin-ajax.php' ) ),
					'siteURL'                            => site_url(),
					'getProText'                         => __( '获取！', 'adaptive-sites' ),
					'getProURL'                          => esc_url( 'https://www.gechiui.com/shop/pro/adaptive-sites/' ),
					'getUpgradeText'                     => __( '升级', 'adaptive-sites' ),
					'getUpgradeURL'                      => esc_url( 'https://www.gechiui.com/shop/pro/adaptive-sites/' ),
					'_ajax_nonce'                        => gc_create_nonce( 'adaptive-sites' ),
					'requiredPlugins'                    => array(),
					'syncLibraryStart'                   => '<span class="message">' . esc_html__( '在后台同步模板库。这个过程可能需要2到3分钟。办完后我们会通知你的。', 'adaptive-sites' ) . '</span>',
					'xmlRequiredFilesMissing'            => __( '导入过程中需要的一些文件丢失了。<br/><br/>请稍后再试。', 'adaptive-sites' ),
					'importFailedMessageDueToDebug'      => __( "<p>GeChiUI调试模式目前已在您的网站上启用。这中断了导入过程。</p><p>请禁用调试模式并重试导入启动器模板。</p><p>您可以将以下代码添加到gc-config.php文件中以禁用调试模式。</p><p><code>define('gc_DEBUG', false);</code></p>", 'adaptive-sites' ),
					/* translators: %s is a documentation link. */
					'importFailedMessage'                => sprintf( __( '<p>在导入此模板时，我们面临一个暂时性问题。</p><p>阅读 <a href="%s" target="_blank">文章</a> 以解决问题并继续导入模板。</p>', 'adaptive-sites' ), esc_url( 'https://docs.gechiui.com/adaptive/fix-starter-template-importing-issues/' ) ),
					/* translators: %s is a documentation link. */
					'importFailedRequiredPluginsMessage' => sprintf( __( '<p>我们在为此模板安装所需的插件时面临暂时问题。</p><p>阅读 <a href="%s" target="_blank">文章</a> 以解决问题并继续导入模板。</p>', 'adaptive-sites' ), esc_url( 'https://docs.gechiui.com/adaptive/plugin-installation-failed-multisite/' ) ),

					'strings'                            => array(
						/* translators: %s are white label strings. */
						'warningBeforeCloseWindow' => sprintf( __( '警告!%1$s的导入过程未完成。在导入过程完成之前，不要关闭窗口。你还想关闭窗口吗?', 'adaptive-sites' ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ),
						'viewSite'                 => __( '完成了!查看网站', 'adaptive-sites' ),
						'syncCompleteMessage'      => self::get_instance()->get_sync_complete_message(),
						/* translators: %s is a template name */
						'importSingleTemplate'     => __( '导入 "%s" 模板', 'adaptive-sites' ),
					),
					'log'                                => array(
						'bulkInstall'  => __( '需要安装插件..', 'adaptive-sites' ),
						/* translators: %s are white label strings. */
						'themeInstall' => sprintf( __( '安装%1$s主题. .', 'adaptive-sites' ), Adaptive_Sites_White_Label::get_instance()->get_option( 'gcoa', 'name', '格尺OA' ) ),
					),
					'default_page_builder'               => $default_page_builder,
					'default_page_builder_data'          => Adaptive_Sites_Page::get_instance()->get_default_page_builder(),
					'default_page_builder_sites'         => Adaptive_Sites_Page::get_instance()->get_sites_by_page_builder( $default_page_builder ),
					'sites'                              => $request_params,
					'categories'                         => array(),
					'page-builders'                      => array(),
					'api_sites_and_pages_tags'           => $this->get_api_option( 'adaptive-sites-tags' ),
					'license_status'                     => $license_status,
					'license_page_builder'               => get_option( 'adaptive-sites-license-page-builder', '' ),

					'ApiDomain'                          => $this->api_domain,
					'ApiURL'                             => $this->api_url,
					'stored_data'                        => $stored_data,
					'favorite_data'                      => $favorite_data,
					'category_slug'                      => 'adaptive-site-category',
					'page_builder'                       => 'adaptive-site-page-builder',
					'cpt_slug'                           => 'adaptive-sites',
					'parent_category'                    => '',
					'compatibilities'                    => $this->get_compatibilities(),
					'compatibilities_data'               => $this->get_compatibilities_data(),
					'dismiss'                            => __( '屏蔽这个通知。', 'adaptive-sites' ),
					'headings'                           => array(
						'subscription' => esc_html__( '最后一步..', 'adaptive-sites' ),
						'site_import'  => esc_html__( '您选择的网站正在被导入。', 'adaptive-sites' ),
						'page_import'  => esc_html__( '您选择的模板将被导入。', 'adaptive-sites' ),
					),
					'subscriptionSuccessMessage'         => esc_html__( '我们已经在您的电子邮件地址上给您发送了惊喜礼物！请检查您的收件箱！', 'adaptive-sites' ),
					'first_import_complete'              => get_option( 'adaptive_sites_import_complete' ),
					'server_import_primary_error'        => __( '看起来您正在导入的模板暂时不可用。', 'adaptive-sites' ),
					'client_import_primary_error'        => __( '我们无法启动导入过程，这是来自GeChiUI的消息：', 'adaptive-sites' ),
					'cloudflare_import_primary_error'    => __( '连接到入门模板 API 时出现错误。', 'adaptive-sites' ),
					'xml_import_interrupted_primary'     => __( '导入内容时出现了错误。', 'adaptive-sites' ),
					'xml_import_interrupted_secondary'   => __( '要导入内容，GeChiUI 需要将 XML 文件存储在 /gc-content/ 文件夹中。请与您的托管服务提供商联系。', 'adaptive-sites' ),
					'xml_import_interrupted_error'       => __( '看起来您的主机可能无法将 XML 文件存储在 /gc-content/ 文件夹中。', 'adaptive-sites' ),
					/* translators: %s HTML tags */
					'ajax_request_failed_primary'        => sprintf( __( '%1$s由于 AJAX 请求失败，我们无法启动导入过程，这是来自 GeChiUI：%2$s', 'adaptive-sites' ), '<p>', '</p>' ),
					/* translators: %s URL to document. */
					'ajax_request_failed_secondary'      => sprintf( __( '%1$s阅读 <a href="%2$s" target="_blank">文章</a> 以解决问题并继续导入模板。%3$s', 'adaptive-sites' ), '<p>', esc_url( 'https://docs.gechiui.com/adaptive/internal-server-error-starter-templates/' ), '</p>' ),
					'cta_links' => $this->get_cta_links(),
					'cta_quick_corner_links' => $this->get_cta_links( 'quick-links-corner' ),
					'cta_premium_popup_links' => $this->get_cta_links( 'get-premium-access-popup' ),
					'cta_link' => $this->get_cta_link(),
					'cta_quick_corner_link' => $this->get_cta_link( 'quick-links-corner' ),
					'cta_premium_popup_link' => $this->get_cta_link( 'get-premium-access-popup' ),

					/* translators: %s URL to document. */
					'process_failed_primary'        => sprintf( __( '%1$s由于 AJAX 请求失败，我们无法完成导入流程，这是消息：%2$s', 'adaptive-sites' ), '<p>', '</p>' ),
					/* translators: %s URL to document. */
					'process_failed_secondary'      => sprintf( __( '%1$s请 <a href="%2$s" target="_blank">在此</a>处报告。%3$s', 'adaptive-sites' ), '<p>', esc_url( 'https://www.gechiui.com/shop/pro/adaptive-sites/' ), '</p>' ),
				)
			);

			return $data;
		}

		/**
		 * Display subscription form
		 *
		 * @since 2.6.1
		 *
		 * @return boolean
		 */
		public function should_display_subscription_form() {

			$subscription = apply_filters( 'adaptive_sites_should_display_subscription_form', null );
			if ( null !== $subscription ) {
				return $subscription;
			}

			// Is WhiteLabel enabled?
			if ( Adaptive_Sites_White_Label::get_instance()->is_white_labeled() ) {
				return false;
			}

			// Is Premium Starter Templates pluign?
			if ( defined( 'ADAPTIVE_SITES_NAME' ) ) {
				return false;
			}

			// User already subscribed?
			$subscribed = get_user_meta( get_current_user_ID(), 'adaptive-sites-subscribed', true );
			if ( $subscribed ) {
				return false;
			}

			return true;
		}

		/**
		 * Import Compatibility Errors
		 *
		 * @since 2.0.0
		 * @return mixed
		 */
		public function get_compatibilities_data() {
			return array(
				'xmlreader'            => array(
					'title'   => esc_html__( 'XMLReader支持失效', 'adaptive-sites' ),
					/* translators: %s doc link. */
					'tooltip' => '<p>' . esc_html__( '您即将导入模板。要完成这个过程，请在您的网站上启用XMLReader支持。', 'adaptive-sites' ) . '</p><p>' . sprintf( __( '请阅读文章 <a href="%s" target="_blank">here</a>来解决这个问题。', 'adaptive-sites' ), 'https://docs.gechiui.com/adaptive/xmlreader-missing/' ) . '</p>',
				),
				'curl'                 => array(
					'title'   => esc_html__( 'cURL 不支持', 'adaptive-sites' ),
					/* translators: %s doc link. */
					'tooltip' => '<p>' . esc_html__( '为了运行一个平滑的导入，请在您的网站上启用cURL支持。', 'adaptive-sites' ) . '</p><p>' . sprintf( __( '请阅读文章 <a href="%s" target="_blank">here</a>来解决这个问题。', 'adaptive-sites' ), 'https://docs.gechiui.com/adaptive/curl-support-missing/' ) . '</p>',
				),
				'gc-debug'             => array(
					'title'   => esc_html__( '禁用调试模式', 'adaptive-sites' ),
					/* translators: %s doc link. */
					'tooltip' => '<p>' . esc_html__( 'GeChiUI调试模式目前已在您的网站上启用。这样，第三方插件的任何错误都可能影响导入过程。', 'adaptive-sites' ) . '</p><p>' . esc_html__( '请禁用它继续导入导入演示数据。为此，您可以将以下代码添加到gc-config.php文件中。', 'adaptive-sites' ) . '</p><p><code>define(\'GC_DEBUG\', false);</code></p><p>' . sprintf( __( '请阅读文章 <a href="%s" target="_blank">here</a>来解决这个问题。', 'adaptive-sites' ), 'https://docs.gechiui.com/adaptive/disable-debug-mode/' ) . '</p>',
				),
				'update-available'     => array(
					'title'   => esc_html__( '更新插件', 'adaptive-sites' ),
					/* translators: %s update page link. */
					'tooltip' => '<p>' . esc_html__( '在这个初始模板中使用的插件可以得到更新。', 'adaptive-sites' ) . '</p>##LIST##<p>' . sprintf( __( '请<a href="%s" target="_blank">更新</a>这些以便成功导入。跳过这一步可能会破坏模板的设计/特性。', 'adaptive-sites' ), esc_url( network_admin_url( 'update-core.php' ) ) ) . '</p>',
				),
				'third-party-required' => array(
					'title'   => esc_html__( '缺少需要的插件', 'adaptive-sites' ),
					'tooltip' => '<p>' . esc_html__( '这个启动器模板需要高级插件。因为这些是第三方的高级插件，首先你需要购买，安装和激活他们。', 'adaptive-sites' ) . '</p>',
				),
				'dynamic-page'         => array(
					'title'   => esc_html__( '动态页', 'adaptive-sites' ),
					'tooltip' => '<p>' . esc_html__( '您将要导入的页面模板包含一个动态小工具/模块。请注意，导入的页面无法使用此动态数据。', 'adaptive-sites' ) . '</p><p>' . esc_html__( '您需要在页面上手动添加它。', 'adaptive-sites' ) . '</p><p>' . esc_html__( '当您导入整个站点时，该动态内容将可用。', 'adaptive-sites' ) . '</p>',
				),
			);
		}

		/**
		 * Get all compatibilities
		 *
		 * @since 2.0.0
		 *
		 * @return array
		 */
		public function get_compatibilities() {

			$data = $this->get_compatibilities_data();

			$compatibilities = array(
				'errors'   => array(),
				'warnings' => array(),
			);

			if ( defined( 'GC_DEBUG' ) && GC_DEBUG ) {
				$compatibilities['warnings']['gc-debug'] = $data['gc-debug'];
			}

			if ( ! class_exists( 'XMLReader' ) ) {
				$compatibilities['errors']['xmlreader'] = $data['xmlreader'];
			}

			if ( ! function_exists( 'curl_version' ) ) {
				$compatibilities['errors']['curl'] = $data['curl'];
			}

			return $compatibilities;
		}

		/**
		 * Register module required js on elementor's action.
		 *
		 * @since 2.0.0
		 */
		public function register_widget_scripts() {

			$page_builders = self::get_instance()->get_page_builders();
			$has_elementor = false;

			foreach ( $page_builders as $page_builder ) {

				if ( 'elementor' === $page_builder['slug'] ) {
					$has_elementor = true;
				}
			}

			if ( ! $has_elementor ) {
				return;
			}

			gc_enqueue_script( 'adaptive-sites-helper', ADAPTIVE_SITES_URI . 'inc/assets/js/helper.js', array( 'jquery' ), ADAPTIVE_SITES_VER, true );

			gc_enqueue_script( 'masonry' );
			gc_enqueue_script( 'imagesloaded' );

			// Image Search assets.
			$this->image_search_assets();

			gc_enqueue_script( 'adaptive-sites-elementor-admin-page', ADAPTIVE_SITES_URI . 'inc/assets/js/elementor-admin-page.js', array( 'jquery', 'gc-util', 'updates', 'masonry', 'imagesloaded' ), ADAPTIVE_SITES_VER, true );
			gc_add_inline_script( 'adaptive-sites-elementor-admin-page', sprintf( 'var pagenow = "%s";', ADAPTIVE_SITES_NAME ), 'after' );
			gc_enqueue_style( 'adaptive-sites-admin', ADAPTIVE_SITES_URI . 'inc/assets/css/admin.css', ADAPTIVE_SITES_VER, true );
			gc_style_add_data( 'adaptive-sites-admin', 'rtl', 'replace' );

			// Use this for premium demos.
			$request_params = apply_filters(
				'adaptive_sites_api_params',
				array(
					'purchase_key' => '',
					'site_url'     => '',
					'per-page'     => 15,
				)
			);

			$license_status = get_pro_license_valid();


			$data = apply_filters(
				'adaptive_sites_render_localize_vars',
				array(
					'plugin_name'                => Adaptive_Sites_White_Label::get_instance()->get_white_label_name(),
					'sites'                      => $request_params,
					'settings'                   => array(),
					'page-builders'              => array(),
					'categories'                 => array(),
					'default_page_builder'       => 'elementor',
					'adaptive_blocks'               => $this->get_all_blocks(),
					'license_status'             => $license_status,
					'ajaxurl'                    => esc_url( admin_url( 'admin-ajax.php' ) ),
					'api_sites_and_pages_tags'   => $this->get_api_option( 'adaptive-sites-tags' ),
					'default_page_builder_sites' => Adaptive_Sites_Page::get_instance()->get_sites_by_page_builder( 'elementor' ),
					'ApiURL'                     => $this->api_url,
					'_ajax_nonce'                => gc_create_nonce( 'adaptive-sites' ),
					'isPro'                      => defined( 'ADAPTIVE_SITES_NAME' ) ? true : false,
					'license_msg'                => '',
					'isWhiteLabeled'             => Adaptive_Sites_White_Label::get_instance()->is_white_labeled(),
					'getProText'                 => __( '获取！', 'adaptive-sites' ),
					'getProURL'                  => esc_url( 'https://www.gechiui.com/shop/pro/adaptive-sites/' ),
					'adaptive_block_categories'     => $this->get_api_option( 'adaptive-blocks-categories' ),
					'siteURL'                    => site_url(),
					'template'                   => esc_html__( '模板', 'adaptive-sites' ),
					'block'                      => esc_html__( '块', 'adaptive-sites' ),
					'dismiss_text'               => esc_html__( '屏蔽', 'adaptive-sites' ),
					'install_plugin_text'        => esc_html__( '需要安装插件', 'adaptive-sites' ),
					'syncCompleteMessage'        => self::get_instance()->get_sync_complete_message(),
					/* translators: %s are link. */
					'page_settings'              => array(
						'message'  => __( '您可以找到<strong>导入演示数据设置</strong>在Style选项卡的<strong>Page Settings</strong>下。', 'adaptive-sites' ),
						'url'      => '#',
						'url_text' => __( '查看全文 →', 'adaptive-sites' ),
					),
				)
			);

			gc_localize_script( 'adaptive-sites-elementor-admin-page', 'adaptiveElementorSites', $data );
		}

		/**
		 * Register module required js on elementor's action.
		 *
		 * @since 2.0.0
		 */
		public function popup_styles() {

			gc_enqueue_style( 'adaptive-sites-elementor-admin-page', ADAPTIVE_SITES_URI . 'inc/assets/css/elementor-admin.css', ADAPTIVE_SITES_VER, true );
			gc_enqueue_style( 'adaptive-sites-elementor-admin-page-dark', ADAPTIVE_SITES_URI . 'inc/assets/css/elementor-admin-dark.css', ADAPTIVE_SITES_VER, true );
			gc_style_add_data( 'adaptive-sites-elementor-admin-page', 'rtl', 'replace' );

		}

		/**
		 * Get all sites
		 *
		 * @since 2.0.0
		 * @return array All sites.
		 */
		public function get_all_sites() {
			$sites_and_pages = array();
			$total_requests  = (int) get_site_option( 'adaptive-sites-requests', 0 );

			for ( $page = 1; $page <= $total_requests; $page++ ) {
				$current_page_data = get_site_option( 'adaptive-sites-and-pages-page-' . $page, array() );
				if ( ! empty( $current_page_data ) ) {
					foreach ( $current_page_data as $page_id => $page_data ) {
						$sites_and_pages[ $page_id ] = $page_data;
					}
				}
			}

			return $sites_and_pages;
		}

		/**
		 * Get all sites
		 *
		 * @since 2.2.4
		 * @param  array $option Site options name.
		 * @return array Site Option value.
		 */
		public function get_api_option( $option ) {
			return get_site_option( $option, array() );
		}

		/**
		 * Get all blocks
		 *
		 * @since 2.0.0
		 * @return array All Elementor Blocks.
		 */
		public function get_all_blocks() {

			$blocks         = array();
			$total_requests = (int) get_site_option( 'adaptive-blocks-requests', 0 );

			for ( $page = 1; $page <= $total_requests; $page++ ) {
				$current_page_data = get_site_option( 'adaptive-blocks-' . $page, array() );
				if ( ! empty( $current_page_data ) ) {
					foreach ( $current_page_data as $page_id => $page_data ) {
						$blocks[ $page_id ] = $page_data;
					}
				}
			}

			return $blocks;
		}

		/**
		 * Load all the required files in the importer.
		 *
		 * @since  1.0.0
		 */
		private function includes() {

			require_once ADAPTIVE_SITES_DIR . 'inc/classes/functions.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-update.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-white-label.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-page.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-elementor-pages.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-elementor-images.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/compatibility/class-adaptive-sites-compatibility.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-importer.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/class-adaptive-sites-gc-cli.php';
			require_once ADAPTIVE_SITES_DIR . 'inc/lib/class-adaptive-sites-ast-block-templates.php';

			// Batch Import.
			require_once ADAPTIVE_SITES_DIR . 'inc/classes/batch-import/class-adaptive-sites-batch-import.php';
		}

		/**
		 * Required Plugin Activate
		 *
		 * @since 2.0.0 Added parameters $init, $options & $enabled_extensions to add the GC CLI support.
		 * @since 1.0.0
		 * @param  string $init               Plugin init file.
		 * @param  array  $options            Site options.
		 * @param  array  $enabled_extensions Enabled extensions.
		 * @return void
		 */
		public function required_plugin_activate( $init = '', $options = array(), $enabled_extensions = array() ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'install_plugins' ) || ! isset( $_POST['init'] ) || ! $_POST['init'] ) {
					gc_send_json_error(
						array(
							'success' => false,
							'message' => __( '错误:您没有安装插件所需的权限。', 'adaptive-sites' ),
						)
					);
				}
			}

			$plugin_init = ( isset( $_POST['init'] ) ) ? esc_attr( $_POST['init'] ) : $init;

			gc_clean_plugins_cache();

			$activate = activate_plugin( $plugin_init, '', false, true );

			if ( is_gc_error( $activate ) ) {
				if ( defined( 'GC_CLI' ) ) {
					GC_CLI::error( '插件激活错误: ' . $activate->get_error_message() );
				} elseif ( gc_doing_ajax() ) {
					gc_send_json_error(
						array(
							'success' => false,
							'message' => $activate->get_error_message(),
						)
					);
				}
			}

			$options = adaptive_get_site_data( 'adaptive-site-options-data' );
			$enabled_extensions = adaptive_get_site_data( 'adaptive-enabled-extensions' );

			$this->after_plugin_activate( $plugin_init, $options, $enabled_extensions );

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( '插件已激活!' );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success(
					array(
						'success' => true,
						'message' => __( '插件激活', 'adaptive-sites' ),
					)
				);
			}
		}

		/**
		 * 必需的主题激活
		 *
		 * @since 2.0.0 Added parameters $init, $options & $enabled_extensions to add the GC CLI support.
		 * @since 1.0.0
		 * @param  string $init               Theme init file.
		 * @param  array  $options            Site options.
		 * @param  array  $enabled_extensions Enabled extensions.
		 * @return void
		 */
		public function required_theme_activate( $slug = '', $options = array(), $enabled_extensions = array() ) {

			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );

				if ( ! current_user_can( 'install_themes' ) ) {
					gc_send_json_error(
						array(
							'success' => false,
							'message' => __( '错误:您没有安装主题所需的权限。', 'adaptive-sites' ),
						)
					);
				}
			}

			$theme_slug = ( isset( $_POST['slug'] ) ) ? esc_attr( $_POST['slug'] ) : $slug;

			// gc_clean_themes_cache();


			// $activate = activate_theme( $theme_slug, '', false, true );

			// if ( is_gc_error( $activate ) ) {
			// 	if ( defined( 'GC_CLI' ) ) {
			// 		GC_CLI::error( '主题激活错误: ' . $activate->get_error_message() );
			// 	} elseif ( gc_doing_ajax() ) {
			// 		gc_send_json_error(
			// 			array(
			// 				'success' => false,
			// 				'message' => $activate->get_error_message(),
			// 			)
			// 		);
			// 	}
			// }

			// $options = adaptive_get_site_data( 'adaptive-site-options-data' );
			// $enabled_extensions = adaptive_get_site_data( 'adaptive-enabled-extensions' );

			// $this->after_theme_activate( $theme_init, $options, $enabled_extensions );

			switch_theme( $theme_slug );

			if ( defined( 'GC_CLI' ) ) {
				GC_CLI::line( '主题已激活!' );
			} elseif ( gc_doing_ajax() ) {
				gc_send_json_success(
					array(
						'success' => true,
						'message' => __( '主题激活', 'adaptive-sites' ),
					)
				);
			}
		}

		/**
		 * Required Plugins
		 *
		 * @since 2.0.0
		 *
		 * @param  array $required_plugins Required Plugins.
		 * @param  array $options            Site Options.
		 * @param  array $enabled_extensions Enabled Extensions.
		 * @return mixed
		 */
		public function required_plugin( $required_plugins = array(), $options = array(), $enabled_extensions = array() ) {

			// Verify Nonce.
			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );
			}

			$response = array(
				'active'       => array(),
				'inactive'     => array(),
				'notinstalled' => array(),
			);

			$options = adaptive_get_site_data( 'adaptive-site-options-data' );
			$enabled_extensions = adaptive_get_site_data( 'adaptive-enabled-extensions' );
			$required_plugins = adaptive_get_site_data( 'required-plugins' );
			$learndash_course_grid = 'https://www.learndash.com/add-on/course-grid/';
			$learndash_woocommerce = 'https://www.learndash.com/add-on/woocommerce/';
			if ( is_plugin_active( 'sfwd-lms/sfwd_lms.php' ) ) {
				$learndash_addons_url  = admin_url( 'admin.php?page=learndash_lms_addons' );
				$learndash_course_grid = $learndash_addons_url;
				$learndash_woocommerce = $learndash_addons_url;
			}

			$third_party_required_plugins = array();
			$third_party_plugins          = array(
				'sfwd-lms'              => array(
					'init' => 'sfwd-lms/sfwd_lms.php',
					'name' => 'LearnDash LMS',
					'link' => 'https://www.learndash.com/',
				),
				'learndash-course-grid' => array(
					'init' => 'learndash-course-grid/learndash_course_grid.php',
					'name' => 'LearnDash Course Grid',
					'link' => $learndash_course_grid,
				),
				'learndash-woocommerce' => array(
					'init' => 'learndash-woocommerce/learndash_woocommerce.php',
					'name' => 'LearnDash WooCommerce Integration',
					'link' => $learndash_woocommerce,
				),
			);

			$plugin_updates          = get_plugin_updates();
			$update_avilable_plugins = array();

			if ( ! empty( $required_plugins ) ) {
				foreach ( $required_plugins as $key => $plugin ) {

					$plugin = (array) $plugin;

					/**
					 * Has Pro Version Support?
					 * And
					 * Is Pro Version Installed?
					 */
					$plugin_pro = $this->pro_plugin_exist( $plugin['init'] );
					if ( $plugin_pro ) {

						if ( array_key_exists( $plugin_pro['init'], $plugin_updates ) ) {
							$update_avilable_plugins[] = $plugin_pro;
						}

						// Pro - Active.
						if ( is_plugin_active( $plugin_pro['init'] ) ) {
							$response['active'][] = $plugin_pro;

							$this->after_plugin_activate( $plugin['init'], $options, $enabled_extensions );

							// Pro - Inactive.
						} else {
							$response['inactive'][] = $plugin_pro;
						}
					} else {
						if ( array_key_exists( $plugin['init'], $plugin_updates ) ) {
							$update_avilable_plugins[] = $plugin;
						}

						// Lite - Installed but Inactive.
						if ( file_exists( GC_PLUGIN_DIR . '/' . $plugin['init'] ) && is_plugin_inactive( $plugin['init'] ) ) {

							$response['inactive'][] = $plugin;

							// Lite - Not Installed.
						} elseif ( ! file_exists( GC_PLUGIN_DIR . '/' . $plugin['init'] ) ) {

							// Added premium plugins which need to install first.
							if ( array_key_exists( $plugin['slug'], $third_party_plugins ) ) {
								$third_party_required_plugins[] = $third_party_plugins[ $plugin['slug'] ];
							} else {
								$response['notinstalled'][] = $plugin;
							}

							// Lite - Active.
						} else {
							$response['active'][] = $plugin;

							$this->after_plugin_activate( $plugin['init'], $options, $enabled_extensions );
						}
					}
				}
			}

			// Checking the `install_plugins` and `activate_plugins` capability for the current user.
			// To perform plugin installation process.
			if (
				( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) &&
				( ( ! current_user_can( 'install_plugins' ) && ! empty( $response['notinstalled'] ) ) || ( ! current_user_can( 'activate_plugins' ) && ! empty( $response['inactive'] ) ) ) ) {
				$message               = __( '权限不足。请联系你的超级管理员允许安装所需的插件权限。', 'adaptive-sites' );
				$required_plugins_list = array_merge( $response['notinstalled'], $response['inactive'] );
				$markup                = $message;
				$markup               .= '<ul>';
				foreach ( $required_plugins_list as $key => $required_plugin ) {
					$markup .= '<li>' . esc_html( $required_plugin['name'] ) . '</li>';
				}
				$markup .= '</ul>';

				gc_send_json_error( $markup );
			}

			$data = array(
				'required_plugins'             => $response,
				'third_party_required_plugins' => $third_party_required_plugins,
				'update_avilable_plugins'      => $update_avilable_plugins,
			);

			if ( gc_doing_ajax() ) {
				gc_send_json_success( $data );
			} else {
				return $data;
			}
		}


		/**
		 * 所需主题
		 *
		 * @since 2.0.0
		 *
		 * @param  array $required_themes Required Themes.
		 * @param  array $options            Site Options.
		 * @param  array $enabled_extensions Enabled Extensions.
		 * @return mixed
		 */
		public function required_theme( $required_themes = array(), $options = array(), $enabled_extensions = array() ) {

			// Verify Nonce.
			if ( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) {
				check_ajax_referer( 'adaptive-sites', '_ajax_nonce' );
			}

			$response = array(
				'active'       => array(),
				'inactive'     => array(),
				'notinstalled' => array(),
			);

			$options = adaptive_get_site_data( 'adaptive-site-options-data' );
			// $enabled_extensions = adaptive_get_site_data( 'adaptive-enabled-extensions' );
			$required_themes = adaptive_get_site_data( 'required-themes' );
			// $learndash_course_grid = 'https://www.learndash.com/add-on/course-grid/';
			// $learndash_woocommerce = 'https://www.learndash.com/add-on/woocommerce/';
			// if ( is_theme_active( 'sfwd-lms/sfwd_lms.php' ) ) {
			// 	$learndash_addons_url  = admin_url( 'admin.php?page=learndash_lms_addons' );
			// 	$learndash_course_grid = $learndash_addons_url;
			// 	$learndash_woocommerce = $learndash_addons_url;
			// }

			// $third_party_required_themes = array();
			// $third_party_themes          = array();

			$theme_updates          = get_theme_updates();

			$update_avilable_themes = array();

			if ( ! empty( $required_themes ) ) {
				// Theme installed but not activate.
				foreach ( $required_themes as $key => $theme ) {

					$theme = (array) $theme;

					// foreach ( (array) gc_get_themes() as $theme_dir => $gc_theme ) {
					// 	if ( $theme['slug'] === $gc_theme->Template ) {
					// 		return 'installed-but-inactive';
					// 	}
					// }

					if ( array_key_exists( $theme['slug'], $theme_updates ) ) {
						$update_avilable_themes[] = $theme;
					}

					$active_theme = gc_get_theme();

					if( $active_theme->Template == $theme['slug']){
						$response['active'][] = $theme;
					}elseif(file_exists( get_theme_root() . '/' . $theme['init'])){
						$response['inactive'][] = $theme;
					}else{
						$response['notinstalled'][] = $theme;
					}
				}
			}




			// Checking the `install_themes` and `activate_themes` capability for the current user.
			// To perform theme installation process.
			if (
				( ! defined( 'GC_CLI' ) && gc_doing_ajax() ) &&
				( ( ! current_user_can( 'install_themes' ) && ! empty( $response['notinstalled'] ) ) ) ) {
				$message               = __( '权限不足。请联系你的超级管理员允许安装所需的插件权限。', 'adaptive-sites' );
				$required_themes_list = array_merge( $response['notinstalled'], $response['inactive'] );
				$markup                = $message;
				$markup               .= '<ul>';
				foreach ( $required_themes_list as $key => $required_theme ) {
					$markup .= '<li>' . esc_html( $required_theme['name'] ) . '</li>';
				}
				$markup .= '</ul>';

				gc_send_json_error( $markup );
			}

			$data = array(
				'required_themes'             => $response,
				// 'third_party_required_themes' => $third_party_required_themes,
				'update_avilable_themes'      => $update_avilable_themes,
			);

			if ( gc_doing_ajax() ) {
				gc_send_json_success( $data );
			} else {
				return $data;
			}
		}

		/**
		 * After Plugin Activate
		 *
		 * @since 2.0.0
		 *
		 * @param  string $plugin_init        Plugin Init File.
		 * @param  array  $options            Site Options.
		 * @param  array  $enabled_extensions Enabled Extensions.
		 * @return void
		 */
		public function after_plugin_activate( $plugin_init = '', $options = array(), $enabled_extensions = array() ) {
			$data = array(
				'adaptive_site_options' => $options,
				'enabled_extensions' => $enabled_extensions,
			);

			do_action( 'adaptive_sites_after_plugin_activation', $plugin_init, $data );
		}

		/**
		 * Has Pro Version Support?
		 * And
		 * Is Pro Version Installed?
		 *
		 * Check Pro plugin version exist of requested plugin lite version.
		 *
		 * Eg. If plugin 'BB Lite Version' required to import demo. Then we check the 'BB Agency Version' is exist?
		 * If yes then we only 'Activate' Agency Version. [We couldn't install agency version.]
		 * Else we 'Activate' or 'Install' Lite Version.
		 *
		 * @since 1.0.1
		 *
		 * @param  string $lite_version Lite version init file.
		 * @return mixed               Return false if not installed or not supported by us
		 *                                    else return 'Pro' version details.
		 */
		public function pro_plugin_exist( $lite_version = '' ) {

			// Lite init => Pro init.
			$plugins = apply_filters(
				'adaptive_sites_pro_plugin_exist',
				array(
					'beaver-builder-lite-version/fl-builder.php' => array(
						'slug' => 'bb-plugin',
						'init' => 'bb-plugin/fl-builder.php',
						'name' => 'Beaver Builder Plugin',
					),
					'ultimate-addons-for-beaver-builder-lite/bb-ultimate-addon.php' => array(
						'slug' => 'bb-ultimate-addon',
						'init' => 'bb-ultimate-addon/bb-ultimate-addon.php',
						'name' => 'Ultimate Addon for Beaver Builder',
					),
					'gcforms-lite/gcforms.php' => array(
						'slug' => 'gcforms',
						'init' => 'gcforms/gcforms.php',
						'name' => 'GCForms',
					),
				),
				$lite_version
			);

			if ( isset( $plugins[ $lite_version ] ) ) {

				// Pro plugin directory exist?
				if ( file_exists( GC_PLUGIN_DIR . '/' . $plugins[ $lite_version ]['init'] ) ) {
					return $plugins[ $lite_version ];
				}
			}

			return false;
		}

		/**
		 * Get Default Page Builders
		 *
		 * @since 2.0.0
		 * @return array
		 */
		public function get_default_page_builders() {
			return array(
				array(
					'id'   => 42,
					'slug' => 'gutenberg',
					'name' => __( '区块构建器', 'adaptive-sites' ),
				),
				array(
					'id'   => 33,
					'slug' => 'elementor',
					'name' => __( '元素构建器', 'adaptive-sites' ),
				),
			);
		}

		/**
		 * Get Page Builders
		 *
		 * @since 2.0.0
		 * @return array
		 */
		public function get_page_builders() {
			return $this->get_default_page_builders();
		}

		/**
		 * Get Page Builder Filed
		 *
		 * @since 2.0.0
		 * @param  string $page_builder Page Bulider.
		 * @param  string $field        Field name.
		 * @return mixed
		 */
		public function get_page_builder_field( $page_builder = '', $field = '' ) {
			if ( empty( $page_builder ) ) {
				return '';
			}

			$page_builders = self::get_instance()->get_page_builders();
			if ( empty( $page_builders ) ) {
				return '';
			}

			foreach ( $page_builders as $key => $current_page_builder ) {
				if ( $page_builder === $current_page_builder['slug'] ) {
					if ( isset( $current_page_builder[ $field ] ) ) {
						return $current_page_builder[ $field ];
					}
				}
			}

			return '';
		}

		/**
		 * Get License Key
		 *
		 * @since 2.0.0
		 * @return array
		 */
		public function get_license_key() {
			if ( class_exists( 'BSF_License_Manager' ) ) {
				if ( BSF_License_Manager::bsf_is_active_license( 'adaptive-sites' ) ) {
					return BSF_License_Manager::instance()->bsf_get_product_info( 'adaptive-sites', 'purchase_key' );
				}
			}

			return '';
		}

		/**
		 * Get Sync Complete Message
		 *
		 * @since 2.0.0
		 * @param  boolean $echo Echo the message.
		 * @return mixed
		 */
		public function get_sync_complete_message( $echo = false ) {

			$message = __( '模板库刷新!', 'adaptive-sites' );

			if ( $echo ) {
				echo esc_html( $message );
			} else {
				return esc_html( $message );
			}
		}

		/**
		 * Get an instance of GC_Filesystem_Direct.
		 *
		 * @since 2.0.0
		 * @return object A GC_Filesystem_Direct instance.
		 */
		public static function get_filesystem() {
			global $gc_filesystem;

			require_once ABSPATH . '/gc-admin/includes/file.php';

			GC_Filesystem();

			return $gc_filesystem;
		}
	}

	/**
	 * Kicking this off by calling 'get_instance()' method
	 */
	Adaptive_Sites::get_instance();

endif;
