<?php
/**
 * Batch Processing
 *
 * @package Adaptive Sites
 * @since 2.0.0
 */

if ( ! class_exists( 'Adaptive_Sites_Batch_Processing_Importer' ) ) :

	/**
	 * Adaptive_Sites_Batch_Processing_Importer
	 *
	 * @since 1.0.14
	 */
	class Adaptive_Sites_Batch_Processing_Importer {

		/**
		 * Instance
		 *
		 * @since 1.0.14
		 * @access private
		 * @var object Class object.
		 */
		private static $instance;

		/**
		 * Initiator
		 *
		 * @since 1.0.14
		 * @return object initialized object of class.
		 */
		public static function get_instance() {
			if ( ! isset( self::$instance ) ) {
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * Constructor
		 *
		 * @since 1.0.14
		 */
		public function __construct() {
		}

		/**
		 * Import Categories
		 *
		 * @since 2.0.0
		 * @return void
		 */
		public function import_categories() {
			adaptive_sites_error_log( 'Requesting Tags' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Requesting Tags', 'no' );

			$api_args     = array(
				'timeout' => 30,
			);
			$tags_request = gc_remote_get( trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/gc/v2/adaptive-sites-tag/?_fields=id,name,slug', $api_args );
			if ( ! is_gc_error( $tags_request ) && 200 === (int) gc_remote_retrieve_response_code( $tags_request ) ) {
				$tags = json_decode( gc_remote_retrieve_body( $tags_request ), true );

				if ( isset( $tags['code'] ) ) {
					$message = isset( $tags['message'] ) ? $tags['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					update_site_option( 'adaptive-sites-tags', $tags, 'no' );

					do_action( 'adaptive_sites_sync_tags', $tags );
				}
			}

			adaptive_sites_error_log( 'Tags Imported Successfully!' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Tags Imported Successfully!', 'no' );
		}

		/**
		 * Import Categories
		 *
		 * @since 2.0.0
		 * @return void
		 */
		public function import_site_categories() {
			adaptive_sites_error_log( 'Requesting Site Categories' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Requesting Site Categories', 'no' );

			$api_args           = array(
				'timeout' => 30,
			);
			$categories_request = gc_remote_get( trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/gc/v2/adaptive-site-category/?_fields=id,name,slug&per_page=100', $api_args );
			if ( ! is_gc_error( $categories_request ) && 200 === (int) gc_remote_retrieve_response_code( $categories_request ) ) {
				$categories = json_decode( gc_remote_retrieve_body( $categories_request ), true );

				if ( isset( $categories['code'] ) ) {
					$message = isset( $categories['message'] ) ? $categories['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					update_site_option( 'adaptive-sites-categories', $categories, 'no' );

					do_action( 'adaptive_sites_sync_categories', $categories );
				}
			}

			adaptive_sites_error_log( 'Site Categories Imported Successfully!' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Site Categories Imported Successfully!', 'no' );
		}

		/**
		 * Import Block Categories
		 *
		 * @since 2.0.0
		 * @return void
		 */
		public function import_block_categories() {
			adaptive_sites_error_log( 'Requesting Block Categories' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Requesting Block Categories', 'no' );

			$api_args     = array(
				'timeout' => 30,
			);
			$tags_request = gc_remote_get( trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/gc/v2/blocks-category/?_fields=id,name,slug&per_page=100&hide_empty=1', $api_args );
			if ( ! is_gc_error( $tags_request ) && 200 === (int) gc_remote_retrieve_response_code( $tags_request ) ) {
				$tags = json_decode( gc_remote_retrieve_body( $tags_request ), true );

				if ( isset( $tags['code'] ) ) {
					$message = isset( $tags['message'] ) ? $tags['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					$categories = array();
					foreach ( $tags as $key => $value ) {
						$categories[ $value['id'] ] = $value;
					}

					update_site_option( 'adaptive-blocks-categories', $categories, 'no' );

					do_action( 'adaptive_sites_sync_blocks_categories', $categories );
				}
			}

			adaptive_sites_error_log( 'Block Categories Imported Successfully!' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Categories Imported Successfully!', 'no' );
		}


		/**
		 * Import Page Builders
		 *
		 * @since 2.0.0
		 * @return void
		 */
		public function set_license_page_builder() {

			adaptive_sites_error_log( 'Requesting License Page Builders' );

			$url = add_query_arg(
				array(
					'_fields'                  => 'id,name,slug',
					'site_url'                 => get_site_url(),
					'purchase_key'             => Adaptive_Sites::get_instance()->get_license_key(),
					'only_allow_page_builders' => 'true',
				),
				trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/gc/v2/adaptive-site-page-builder/'
			);

			$api_args = array(
				'timeout' => 30,
			);

			$page_builder_request = gc_remote_get( $url, $api_args );
			if ( ! is_gc_error( $page_builder_request ) && 200 === (int) gc_remote_retrieve_response_code( $page_builder_request ) ) {
				$page_builders = json_decode( gc_remote_retrieve_body( $page_builder_request ), true );
				if ( isset( $page_builders['code'] ) ) {
					$message = isset( $page_builders['message'] ) ? $page_builders['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					// @codingStandardsIgnoreStart
					// Set mini agency page builder.
					$page_builder_slugs = gc_list_pluck( $page_builders, 'slug' );
					if ( in_array( 'elementor', $page_builder_slugs ) && ! in_array( 'beaver-builder', $page_builder_slugs ) ) {
						update_option( 'adaptive-sites-license-page-builder', 'elementor', 'no' );
						adaptive_sites_error_log( 'Set "Elementor" as License Page Builder' );
					} elseif ( in_array( 'beaver-builder', $page_builder_slugs ) && ! in_array( 'elementor', $page_builder_slugs ) ) {
						update_option( 'adaptive-sites-license-page-builder', 'beaver-builder', 'no' );
						adaptive_sites_error_log( 'Set "Beaver Builder" as License Page Builder' );
					} else {
						update_option( 'adaptive-sites-license-page-builder', '', 'no' );
						adaptive_sites_error_log( 'Not Set Any License Page Builder' );
					}
					// @codingStandardsIgnoreEnd
				}
			}
		}

		/**
		 * Import Page Builders
		 *
		 * @since 2.0.0
		 * @return void
		 */
		public function import_page_builders() {
			adaptive_sites_error_log( 'Requesting Page Builders' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Requesting Page Builders', 'no' );

			$purchase_key = Adaptive_Sites::get_instance()->get_license_key();
			$site_url     = get_site_url();

			$api_args = array(
				'timeout' => 30,
			);

			$page_builder_request = gc_remote_get( trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/gc/v2/adaptive-site-page-builder/?_fields=id,name,slug&site_url=' . $site_url . '&purchase_key=' . $purchase_key, $api_args );
			if ( ! is_gc_error( $page_builder_request ) && 200 === (int) gc_remote_retrieve_response_code( $page_builder_request ) ) {
				$page_builders = json_decode( gc_remote_retrieve_body( $page_builder_request ), true );

				if ( isset( $page_builders['code'] ) ) {
					$message = isset( $page_builders['message'] ) ? $page_builders['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					update_site_option( 'adaptive-sites-page-builders', $page_builders, 'no' );

					do_action( 'adaptive_sites_sync_page_builders', $page_builders );
				}
			}

			adaptive_sites_error_log( 'Page Builders Imported Successfully!' );
			update_site_option( 'adaptive-sites-batch-status-string', 'Page Builders Imported Successfully!', 'no' );
		}

		/**
		 * Import Blocks
		 *
		 * @since 2.0.0
		 * @param  integer $page Page number.
		 * @return void
		 */
		public function import_blocks( $page = 1 ) {

			adaptive_sites_error_log( 'BLOCK: -------- ACTUAL IMPORT --------' );
			$api_args   = array(
				'timeout' => 30,
			);
			$all_blocks = array();
			adaptive_sites_error_log( 'BLOCK: Requesting ' . $page );
			update_site_option( 'adaptive-blocks-batch-status-string', 'Requesting for blocks page - ' . $page, 'no' );

			$query_args = apply_filters(
				'adaptive_sites_blocks_query_args',
				array(
					'page_builder' => 'elementor',
					'per_page'     => 100,
					'page'         => $page,
				)
			);

			$api_url = add_query_arg( $query_args, trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/adaptive-blocks/v1/blocks/' );

			$response = gc_remote_get( $api_url, $api_args );

			if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
				$adaptive_blocks = json_decode( gc_remote_retrieve_body( $response ), true );

				if ( isset( $adaptive_blocks['code'] ) ) {
					$message = isset( $adaptive_blocks['message'] ) ? $adaptive_blocks['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					adaptive_sites_error_log( 'BLOCK: Storing data for page ' . $page . ' in option adaptive-blocks-' . $page );
					update_site_option( 'adaptive-blocks-batch-status-string', 'Storing data for page ' . $page . ' in option adaptive-blocks-' . $page, 'no' );

					update_site_option( 'adaptive-blocks-' . $page, $adaptive_blocks, 'no' );

					do_action( 'adaptive_sites_sync_blocks', $page, $adaptive_blocks );
				}
			} else {
				adaptive_sites_error_log( 'BLOCK: API Error: ' . $response->get_error_message() );
			}

			adaptive_sites_error_log( 'BLOCK: Complete storing data for blocks ' . $page );
			update_site_option( 'adaptive-blocks-batch-status-string', 'Complete storing data for page ' . $page, 'no' );
		}

		/**
		 * Import
		 *
		 * @since 1.0.14
		 * @since 2.0.0 Added page no.
		 *
		 * @param  integer $page Page number.
		 * @return array
		 */
		public function import_sites( $page = 1 ) {
			$api_args        = array(
				'timeout' => 30,
			);
			$sites_and_pages = array();
			adaptive_sites_error_log( 'Requesting ' . $page );
			update_site_option( 'adaptive-sites-batch-status-string', 'Requesting ' . $page, 'no' );

			$query_args = apply_filters(
				'adaptive_sites_import_sites_query_args',
				array(
					'per_page' => 15,
					'page'     => $page,
				)
			);

			$api_url = add_query_arg( $query_args, trailingslashit( Adaptive_Sites::get_instance()->get_api_domain() ) . 'gc-json/adaptive-sites/v1/sites-and-pages/' );

			$response = gc_remote_get( $api_url, $api_args );
			if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
				$sites_and_pages = json_decode( gc_remote_retrieve_body( $response ), true );

				if ( isset( $sites_and_pages['code'] ) ) {
					$message = isset( $sites_and_pages['message'] ) ? $sites_and_pages['message'] : '';
					if ( ! empty( $message ) ) {
						adaptive_sites_error_log( 'HTTP Request Error: ' . $message );
					} else {
						adaptive_sites_error_log( 'HTTP Request Error!' );
					}
				} else {
					adaptive_sites_error_log( 'Storing data for page ' . $page . ' in option adaptive-sites-and-pages-page-' . $page );
					update_site_option( 'adaptive-sites-batch-status-string', 'Storing data for page ' . $page . ' in option adaptive-sites-and-pages-page-' . $page, 'no' );

					update_site_option( 'adaptive-sites-and-pages-page-' . $page, $sites_and_pages, 'no' );

					do_action( 'adaptive_sites_sync_sites_and_pages', $page, $sites_and_pages );
				}
			} else {
				adaptive_sites_error_log( 'API Error: ' . $response->get_error_message() );
			}

			adaptive_sites_error_log( 'Complete storing data for page ' . $page );
			update_site_option( 'adaptive-sites-batch-status-string', 'Complete storing data for page ' . $page, 'no' );

			return $sites_and_pages;
		}
	}

	/**
	 * Kicking this off by calling 'get_instance()' method
	 */
	Adaptive_Sites_Batch_Processing_Importer::get_instance();

endif;
